# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import GemRB
from GUIDefines import *
from ie_stats import *
from ie_restype import RES_2DA
from ie_slots import *
import GameCheck
import GUICommon
import Spellbook
import CommonTables
import LUSpellSelection
import LUProfsSelection
import LUSkillsSelection

#######################
pc = 0
OldClassName = 0
OldKitName = 0
ClassName = 0
NewMageSpells = 0
NewPriestMask = 0
NewClassId = 0
DualClassTable = 0
#######################
DCMainWindow = 0
DCMainClassButton = 0
DCMainSkillsButton = 0
DCMainDoneButton = 0
DCMainStep = 0
#######################
DCClassWindow = 0
DCClassDoneButton = 0
DCClass = 0
DCClasses = []
#######################
DCProfsWindow = 0
DCProfsDoneButton = 0
#######################
DCSkillsWindow = 0
DCSkillsDoneButton = 0
#######################

def DualClassWindow ():
	"""Opens the dual class selection window."""

	global pc, OldClassName, NewMageSpells, NewPriestMask, NewClassId, OldKitName, DualClassTable
	global DCMainWindow, DCMainClassButton, DCMainDoneButton, DCMainSkillsButton, DCMainStep

	# get our basic globals
	pc = GemRB.GameGetSelectedPCSingle ()
	DCMainStep = 1

	# make sure to nullify old values
	NewPriestMask = 0
	NewMageSpells = 0
	NewClassId = 0
	
	# set up our main window
	DCMainWindow = GemRB.LoadWindow (5, "GUIREC")

	# done button (off)
	DCMainDoneButton = DCMainWindow.GetControl (2)
	DCMainDoneButton.SetText (11973)
	DCMainDoneButton.OnPress (DCMainDonePress)
	DCMainDoneButton.SetState (IE_GUI_BUTTON_DISABLED)

	# cancel button (on)
	DCMainCancelButton = DCMainWindow.GetControl (1)
	DCMainCancelButton.SetText (13727)
	DCMainCancelButton.OnPress (DCMainCancelPress)
	DCMainCancelButton.SetState (IE_GUI_BUTTON_ENABLED)
	DCMainCancelButton.MakeEscape ()

	# class button (on)
	DCMainClassButton = DCMainWindow.GetControl (3)
	DCMainClassButton.SetText (11959)
	DCMainClassButton.OnPress (DCMainClassPress)
	DCMainClassButton.SetState (IE_GUI_BUTTON_ENABLED)
	DCMainClassButton.MakeDefault()

	# skills button (off)
	DCMainSkillsButton = DCMainWindow.GetControl (4)
	DCMainSkillsButton.SetText (17372)
	DCMainSkillsButton.OnPress (DCMainSkillsPress)
	DCMainSkillsButton.SetState (IE_GUI_BUTTON_DISABLED)

	# back button (on)
	DCMainBackButton = DCMainWindow.GetControl (5)
	if GameCheck.IsBG2OrEE ():
		DCMainBackButton.SetText (15416)
	DCMainBackButton.OnPress (DCMainBackPress)
	DCMainBackButton.SetState (IE_GUI_BUTTON_ENABLED)

	# picture of character
	DCMainPictureButton = DCMainWindow.GetControl (6)
	DCMainPictureButton.SetState (IE_GUI_BUTTON_LOCKED)
	DCMainPictureButton.SetFlags (IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_PICTURE, OP_SET)
	DCMainPictureButton.SetPicture (GemRB.GetPlayerPortrait (pc, 0)["Sprite"], "NOPORTMD")

	# text area warning
	DCTextArea = DCMainWindow.GetControl (7)
	DCTextArea.SetText (10811)

	# character name
	DCLabel = DCMainWindow.GetControl (0x10000008)
	DCLabel.SetText (GemRB.GetPlayerName (pc, 0))

	# class name
	Kit = GUICommon.GetKitIndex (pc)
	OldClassName = GUICommon.GetClassRowName (pc)
	if Kit:
		OldKitName = CommonTables.KitList.GetValue (Kit, 0, GTV_STR)
	else:
		OldKitName = OldClassName
	DCLabel = DCMainWindow.GetControl (0x10000009)
	DCLabel.SetText (GUICommon.GetActorClassTitle (pc))

	# get the names of the classes we can dual to
	DualClassTable = GemRB.LoadTable ("dualclas")
	for i in range (DualClassTable.GetColumnCount ()):
		DCClasses.append (DualClassTable.GetColumnName (i))

	# show our window
	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)
	return

def DumpUnusableItems (pc):
	"""Dumps everything in the inventory that is now unusable."""

	SlotTypes = [ SLOT_ARMOUR, SLOT_SHIELD, SLOT_HELM, SLOT_RING, SLOT_CLOAK, SLOT_BOOT, SLOT_AMULET, SLOT_GLOVE, SLOT_BELT, SLOT_ITEM, SLOT_WEAPON, SLOT_QUIVER ]
	for stype in SlotTypes:
		Slots = GemRB.GetSlots (pc, stype)
		if not len(Slots):
			# nothing there
			continue

		for slot in Slots:
			SlotType = GemRB.GetSlotType (slot, pc)
			CREItem = GemRB.GetSlotItem (pc, slot)

			if not GemRB.CanUseItemType (SlotType["Type"], CREItem["ItemResRef"], pc):
				# move it to a free inventory slot by mimicking dragging
				Item = GemRB.GetItem (CREItem["ItemResRef"])
				GemRB.DragItem (pc, slot, Item["ItemIcon"])
				if not GemRB.DropDraggedItem (pc, -3): # ASI_FAILED
					# inventory was probably too full, drop to ground
					GemRB.DropDraggedItem (pc, -2)

def DCMainDonePress ():
	"""Saves our dualclass changes and closes the window.

	This de-activates our old class and readjusts the basic actor stats, such as
	THAC0, spell casting, proficiencies, and so forth, to the new class."""
	import GUIREC
	global ClassName

	# save our proficiencies
	LUProfsSelection.ProfsSave (pc, LUProfsSelection.LUPROFS_TYPE_DUALCLASS)

	# remove old class abilities
	KitIndex = GUICommon.GetKitIndex (pc)
	if KitIndex:
		ABTable = CommonTables.KitList.GetValue (str(KitIndex), "ABILITIES")
	else:
		ABTable = CommonTables.ClassSkills.GetValue (OldClassName, "ABILITIES")
	if ABTable != "*" and GemRB.HasResource (ABTable, RES_2DA, 1):
		GUICommon.RemoveClassAbilities (pc, ABTable, GemRB.GetPlayerStat (pc, IE_LEVEL))

	# remove old class casting
	if not NewMageSpells:
		for i in range (9):
			GemRB.SetMemorizableSpellsCount (pc, 0, IE_SPELL_TYPE_WIZARD, i)
	Spellbook.RemoveKnownSpells (pc, IE_SPELL_TYPE_PRIEST, 1,7, 1)

	# apply our class abilities
	ABTable = CommonTables.ClassSkills.GetValue (ClassName, "ABILITIES")
	if ABTable != "*" and GemRB.HasResource (ABTable, RES_2DA, 1):
		GUICommon.AddClassAbilities (pc, ABTable)

	# learn our new priest spells
	if NewPriestMask:
		Spellbook.LearnPriestSpells (pc, 1, NewPriestMask)
		GemRB.SetMemorizableSpellsCount (pc, 1, IE_SPELL_TYPE_PRIEST, 0)

	# save our thief skills if we have them
	LUSkillsSelection.SkillsSave (pc)

	# NOTE: the ordering here is very deliberate, don't touch!
	# for the dc detection code to work in core, some level and mc bits info has to be there
	# 3 PCF functions (class, kit, level) all need more info
	# testcase: dualclassing and checking how the actionbar evolves

	# mark the actor as dualclassed
	OldMCFlags = GemRB.GetPlayerStat (pc, IE_MC_FLAGS, 1)
	NewMCBit = CommonTables.Classes.GetValue (OldClassName, "MC_WAS_ID", GTV_INT)
	GemRB.SetPlayerStat (pc, IE_MC_FLAGS, OldMCFlags|NewMCBit)

	# calculate new dc/mc class combo
	OldClassId = GemRB.GetPlayerStat (pc, IE_CLASS)
	MultClassId = (1 << (NewClassId-1)) | (1 << (OldClassId-1))
	MultClassId = CommonTables.Classes.FindValue ("MULTI", MultClassId)
	MultClassId = CommonTables.Classes.GetRowName (MultClassId)
	MultClassId = CommonTables.Classes.GetValue (MultClassId, "ID")

	# fake update our levels (no PCF), so the PCF can be ran later on correct values
	# core only looks at the level difference when determining dc class order
	DSOverride = { "old": OldClassId, "new": NewClassId, "mc": MultClassId }
	Level1 = GemRB.GetPlayerStat (pc, IE_LEVEL)
	if GUICommon.IsDualSwap (pc, DSOverride):
		GemRB.SetPlayerStat (pc, IE_LEVEL2, 0, 0)
	else:
		GemRB.SetPlayerStat (pc, IE_LEVEL2, Level1 - 1, 0)
		GemRB.SetPlayerStat (pc, IE_LEVEL, 0, 0)

	# finally set the new class
	GemRB.SetPlayerStat (pc, IE_CLASS, MultClassId)

	# update our levels and xp
	if GUICommon.IsDualSwap (pc, DSOverride):
		GemRB.SetPlayerStat (pc, IE_LEVEL2, 1)
	else:
		GemRB.SetPlayerStat (pc, IE_LEVEL2, Level1, 0)
		GemRB.SetPlayerStat (pc, IE_LEVEL, 1)
	GemRB.SetPlayerStat (pc, IE_XP, 0)

	# END of order matters NOTE

	# new thac0
	ThacoTable = GemRB.LoadTable ("THAC0")
	GemRB.SetPlayerStat (pc, IE_TOHIT, ThacoTable.GetValue (NewClassId-1, 0, GTV_INT))

	# new saves
	SavesTable = CommonTables.Classes.GetValue (ClassName, "SAVE", GTV_STR)
	SavesTable = GemRB.LoadTable (SavesTable)
	for i in range (5):
		GemRB.SetPlayerStat (pc, IE_SAVEVSDEATH+i, SavesTable.GetValue (i, 0))

	# dump any equipped items that are now unusable
	DumpUnusableItems (pc)

	# close our window
	if DCMainWindow:
		DCMainWindow.Close ()
	return

def DCMainCancelPress ():
	"""Revert all changes and close the dual class window."""

	# simulate pressing the back button until we get back to DCMainStep = 1
	# to unset all things from the new class
	while DCMainStep > 1:
		DCMainBackPress ()

	# close our window
	if DCMainWindow:
		DCMainWindow.Close ()

	return

def DCMainBackPress ():
	"""Reverts all current changes and reverts back to the previous step."""
	global DCMainStep, DCClass, NewMageSpells
	global NewPriestMask

	if DCMainStep == 2: # class selected, wait to choose skills
		# disable the skills button and re-enable the class button
		# the class will be reset when the class button is clicked
		DCMainSkillsButton.SetState (IE_GUI_BUTTON_DISABLED)
		DCMainClassButton.SetState (IE_GUI_BUTTON_ENABLED)

		# back a step
		DCMainStep = 1
	elif DCMainStep == 3: # skills selected
		# re-enable our skills button
		DCMainSkillsButton.SetState (IE_GUI_BUTTON_ENABLED)

		# un-learn our spells and skills
		if NewMageSpells:
			Spellbook.RemoveKnownSpells (pc, IE_SPELL_TYPE_WIZARD, 1,1, 1)

		LUSkillsSelection.SkillsNullify ()
		NewPriestMask = 0
		NewMageSpells = 0

		# go back a step
		DCMainStep = 2
	return

def DCMainClassPress ():
	"""Opens the class selection window."""

	global DCClassWindow, DCClassDoneButton, DCClass

	# we default the class back down to -1
	DCClass = -1
	GemRB.SetVar ("DCClass", DCClass)

	# open the window
	DCClassWindow = GemRB.LoadWindow (6, "GUIREC")

	# string refs for the given classes
	DCClassStrings = []
	for classname in DCClasses:
		DCClassStrings.append(CommonTables.Classes.GetValue (classname, "NAME_REF", GTV_INT))

	# setup the class buttons
	for i in range (6):
		# get the button and associate it with the correct var
		DCClassButton = DCClassWindow.GetControl (i+1)
		DCClassButton.OnPress (DCClassSelect)
		DCClassButton.SetVarAssoc ("DCClass", i)
		DCClassButton.SetText (DCClassStrings[i])

		# enable only if we can dual into the given class
		if CanDualInto (i):
			DCClassButton.SetState (IE_GUI_BUTTON_ENABLED)
		else:
			DCClassButton.SetState (IE_GUI_BUTTON_DISABLED)

	# done button (off)
	DCClassDoneButton = DCClassWindow.GetControl (8)
	DCClassDoneButton.SetText (11973)
	DCClassDoneButton.OnPress (DCClassDonePress)
	DCClassDoneButton.SetState (IE_GUI_BUTTON_DISABLED)
	DCClassDoneButton.MakeDefault()

	# back button (on)
	DCClassBackButton = DCClassWindow.GetControl (7)
	DCClassBackButton.SetText (15416)
	DCClassBackButton.OnPress (DCClassBackPress)
	DCClassBackButton.SetState (IE_GUI_BUTTON_ENABLED)
	DCClassBackButton.MakeEscape ()

	# setup the text area with default text
	DCClassTextArea = DCClassWindow.GetControl (9)
	DCClassTextArea.SetText (10949)

	# show the window
	DCClassWindow.ShowModal (MODAL_SHADOW_GRAY)

	return

def DCClassSelect ():
	"""Updates the current class based on the button pressed."""

	global DCClass

	# un-select the old button and save and select the new one
	if DCClass >= 0:
		DCClassButton = DCClassWindow.GetControl (DCClass+1)
		DCClassButton.SetState (IE_GUI_BUTTON_ENABLED)

	# if we clicked the same class twice, turn the done button off (toggled)
	if DCClass == GemRB.GetVar ("DCClass"):
		DCClassDoneButton.SetState (IE_GUI_BUTTON_DISABLED)
		DCClass = -1
		
		# don't need to worry about setting a new text area, as it would be the same
		return

	# save the new class, select it's button, and enable the done button
	DCClass = GemRB.GetVar ("DCClass")
	DCClassButton = DCClassWindow.GetControl (DCClass+1)
	DCClassButton.SetState (IE_GUI_BUTTON_SELECTED)
	DCClassDoneButton.SetState (IE_GUI_BUTTON_ENABLED)

	# all the possible strrefs for the different classes
	DCClassStrings = []
	for classname in DCClasses:
		DCClassStrings.append (CommonTables.Classes.GetValue (classname, "DESC_REF", GTV_INT))

	# update the text are with the new string
	DCClassTextArea = DCClassWindow.GetControl (9)
	DCClassTextArea.SetText (DCClassStrings [DCClass])

	return

def DCClassDonePress ():
	"""Stores the selected class and moves to the next step."""

	global DCMainStep, ClassName, NewClassId

	# unload our class selection window
	if DCClassWindow:
		DCClassWindow.Close ()
	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)

	# enable the skills button and disable the class selection button
	DCMainClassButton.SetState (IE_GUI_BUTTON_DISABLED)
	DCMainSkillsButton.SetState (IE_GUI_BUTTON_ENABLED)

	# save the class
	ClassName = DCClasses[DCClass]
	NewClassId = CommonTables.Classes.GetValue (ClassName, "ID", GTV_INT)

	# set our step to 2 so that the back button knows where we are
	DCMainStep = 2

	return

def CanDualInto (index):
	"""Determines if a given class can be dualed into.

	Index defines the position within the DCClasses list."""

	# make sure index isn't out of range
	if index < 0 or index >= len (DCClasses):
		return 0

	# return 0 if we can't dual into the class
	if not DualClassTable.GetValue (OldKitName, DCClasses[index], GTV_INT):
		return 0

	# make sure we aren't restricted by alignment
	AlignmentTable = GemRB.LoadTable ("alignmnt")
	Alignment = GemRB.GetPlayerStat (pc, IE_ALIGNMENT) # our alignment
	Alignment = CommonTables.Aligns.FindValue (3, Alignment)
	Alignment = CommonTables.Aligns.GetValue (Alignment, 4) # convert the alignment
	if not AlignmentTable.GetValue (DCClasses[index], Alignment, GTV_INT): # check it
		return 0

	# make sure we have the minimum stats required for the next class
	StatTable = GemRB.LoadTable ("abdcdsrq")
	ClassStatIndex = StatTable.GetRowIndex (DCClasses[index])
	for stat in range (6): # loop through each stat
		minimum = StatTable.GetValue (ClassStatIndex, stat)
		name = StatTable.GetColumnName (stat)
		if GemRB.GetPlayerStat (pc, GUICommon.SafeStatEval ("IE_" + name[4:])) < minimum:
			return 0

	# if we made it here, we can dual to the class
	return 1

def DCClassBackPress ():
	"""Unloads the class selection window without making any changes."""
	# close the class window
	if DCClassWindow:
		DCClassWindow.Close ()
	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)
	return

def DCMainSkillsPress ():
	"""Opens the proficiency selection window.

	Then follows the skills proficiency and spells windows, if required."""

	global DCSkillsWindow, DCMainStep, DCHasProfs

	# we're onto the next step
	DCMainSkillsButton.SetState (IE_GUI_BUTTON_DISABLED)
	DCMainStep = 3

	DCOpenProfsWindow ()
	return

def DCOpenProfsWindow ():
	"""Opens the proficiency selection window."""

	global DCProfsWindow, DCProfsDoneButton

	# load up our window and set some basic variables
	DCProfsWindow = GemRB.LoadWindow (15, "GUIREC")
	NewClassId = CommonTables.Classes.GetValue (ClassName, "ID", GTV_INT)
	if GameCheck.IsBG2OrEE ():
		LUProfsSelection.SetupProfsWindow (pc, \
			LUProfsSelection.LUPROFS_TYPE_DUALCLASS, DCProfsWindow, DCProfsRedraw, classid=NewClassId)
	else:
		LUProfsSelection.SetupProfsWindow (pc, \
			LUProfsSelection.LUPROFS_TYPE_DUALCLASS, DCProfsWindow, \
			DCProfsRedraw, [0,0,0], [1,1,1], NewClassId, False, 0)

	# setup the done and cancel
	DCProfsDoneButton = DCProfsWindow.GetControl (76)
	DCProfsDoneButton.SetText (11973)
	DCProfsDoneButton.OnPress (DCProfsDonePress)
	DCProfsDoneButton.SetState (IE_GUI_BUTTON_DISABLED)
	DCProfsDoneButton.MakeDefault()

	DCProfsCancelButton = DCProfsWindow.GetControl (77)
	DCProfsCancelButton.SetText (13727)
	DCProfsCancelButton.OnPress (DCProfsCancelPress)
	DCProfsCancelButton.SetState (IE_GUI_BUTTON_ENABLED)
	DCProfsCancelButton.MakeEscape ()

	# show the window and draw away
	DCProfsWindow.ShowModal (MODAL_SHADOW_GRAY)
	DCProfsRedraw ()
	return

def DCProfsRedraw ():
	"""Redraws the proficiency selection window.

	Called whenever a proficiency is assigned or removed."""

	ProfsPointsLeft = GemRB.GetVar ("ProfsPointsLeft")

	if ProfsPointsLeft == 0:
		DCProfsDoneButton.SetState (IE_GUI_BUTTON_ENABLED)
	else:
		DCProfsDoneButton.SetState (IE_GUI_BUTTON_DISABLED)
	return

def DCProfsDonePress ():
	"""Goes to the next applicable step.

	This is either skill selection, spell selection, or nothing."""

	global NewMageSpells, NewPriestMask

	# check for mage spells and thief skills
	SpellTable = CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL")
	ClericTable = CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL")
	DruidTable = CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL")
	if SpellTable != "*":
		LUSpellSelection.OpenSpellsWindow (pc, SpellTable, 1, 1, 0)
		SpellTable = GemRB.LoadTable (SpellTable)
		GemRB.SetMemorizableSpellsCount (pc, SpellTable.GetValue (0, 0), IE_SPELL_TYPE_WIZARD, 0)
		NewMageSpells = 1
	if ClericTable != "*":
		if not GemRB.HasResource(ClericTable, RES_2DA, 1):
			ClericTable = "MXSPLPRS" # iwd1 doesn't have a DRUIDSPELL column in the table
		# make sure we can cast spells at this level (paladins)
		ClericTable = GemRB.LoadTable (ClericTable)
		if ClericTable.GetRowName (0) == "1":
			NewPriestMask = 0x4000
	elif DruidTable != "*":
		# make sure we can cast spells at this level (rangers)
		if GameCheck.HasTOB ():
			DruidTable = GemRB.LoadTable (DruidTable)
			if DruidTable.GetRowName (0) == "1":
				NewPriestMask = 0x8000
		else:
			NewPriestMask = 0x8000

	# open the thieves selection window
	OpenSkillsWindow ()

	# we can be done now
	DCMainDoneButton.SetState (IE_GUI_BUTTON_ENABLED)

	# close out the profs window (don't assign yet!)
	if DCProfsWindow:
		DCProfsWindow.Close ()
	return

def DCProfsCancelPress ():
	"""Closes the profeciency selection window."""

	# close out the profs window and go back a step
	if DCProfsWindow:
		DCProfsWindow.Close ()
	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)

	DCMainBackPress ()
	return

def OpenSkillsWindow ():
	"""Opens the skills selection window.

	This will allow the selection of thief skills for appropriate classes, or
	auto-assign them for classes that receive skills based on level."""

	global DCSkillsWindow, DCSkillsDoneButton

	DCSkillsWindow = GemRB.LoadWindow (7, "GUIREC")
	if GameCheck.IsBG2OrEE ():
		LUSkillsSelection.SetupSkillsWindow (pc, \
			LUSkillsSelection.LUSKILLS_TYPE_DUALCLASS, DCSkillsWindow, DCSkillsRedraw, classid=NewClassId)
	else:
		LUSkillsSelection.SetupSkillsWindow (pc, \
			LUSkillsSelection.LUSKILLS_TYPE_DUALCLASS, DCSkillsWindow, \
			DCSkillsRedraw, [0,0,0], [1,1,1], NewClassId, False)

	#just go back if we can't assign skills
	if not GemRB.GetVar ("SkillPointsLeft"):
		DCSkillsWindow.Close ()
		return

	# setup the back and done buttons
	BackButton = DCSkillsWindow.GetControl(24)
	BackButton.SetText(15416)
	BackButton.MakeEscape()
	BackButton.OnPress (DCSkillsBackPress)

	DCSkillsDoneButton = DCSkillsWindow.GetControl(25)
	DCSkillsDoneButton.SetText(11973)
	DCSkillsDoneButton.MakeDefault()
	DCSkillsDoneButton.OnPress (DCSkillsDonePress)
	DCSkillsDoneButton.SetState(IE_GUI_BUTTON_DISABLED)

	# setup the default text area
	TextArea = DCSkillsWindow.GetControl(22)
	TextArea.SetText(17248)

	DCSkillsWindow.ShowModal (MODAL_SHADOW_GRAY)
	DCSkillsRedraw ()
	return

def DCSkillsRedraw ():
	"""Redraws the skills selection window.

	Called whenever a skill is assigned or removed."""

	# no points left? we can be done! :)
	DCSkillsLeft = GemRB.GetVar ("SkillPointsLeft")
	if DCSkillsLeft <= 0:
		DCSkillsDoneButton.SetState (IE_GUI_BUTTON_ENABLED)
	else:
		DCSkillsDoneButton.SetState (IE_GUI_BUTTON_DISABLED)

def DCSkillsBackPress ():
	"""Reverts all changes to this points."""

	if DCSkillsWindow:
		DCSkillsWindow.Close ()
	DCMainWindow.ShowModal (MODAL_SHADOW_GRAY)
	LUSkillsSelection.SkillsNullify ()
	DCMainBackPress ()
	return

def DCSkillsDonePress ():
	"""Closes the skills selection window."""

	if DCSkillsWindow:
		DCSkillsWindow.Close ()
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# Options.py - scripts to control options windows mostly from GUIOPT winpack

import GemRB
import GUIOPT
from GUIDefines import *

def OnLoad():
	global OptionsWindow

	MessageBarWindow = GemRB.LoadWindow(0, "GUIOPT")

	CharactersBarWindow = GemRB.LoadWindow(1, "GUIOPT")

	if MessageBarWindow:
		MessageBarWindow.Close ()
	if CharactersBarWindow:
		CharactersBarWindow.Close ()

	OptionsWindow = GemRB.LoadWindow(13, "GUIOPT")

	VersionLabel = OptionsWindow.GetControl(0x1000000B)
	VersionLabel.SetText(GemRB.Version)

	GraphicsButton = OptionsWindow.GetControl(7)
	SoundButton = OptionsWindow.GetControl(8)
	GamePlayButton = OptionsWindow.GetControl(9)
	MoviesButton = OptionsWindow.GetControl(14)
	KeyboardButton = OptionsWindow.GetControl(13)
	ReturnButton = OptionsWindow.GetControl(11)

	GraphicsButton.SetText(17162)
	GraphicsButton.OnPress (GUIOPT.OpenVideoOptionsWindow)
	SoundButton.SetText(17164)
	SoundButton.OnPress (GUIOPT.OpenAudioOptionsWindow)
	GamePlayButton.SetText(17165)
	GamePlayButton.OnPress (GUIOPT.OpenGameplayOptionsWindow)
	MoviesButton.SetText(15415)
	MoviesButton.OnPress (GUIOPT.OpenMovieWindow)
	KeyboardButton.SetText(33468)
	KeyboardButton.OnPress (GUIOPT.OpenHotkeyOptionsWindow)

	ReturnButton.SetText(10308)
	def CloseStartOptions ():
		GemRB.GetView ("STARTWIN").SetDisabled (False)
		OptionsWindow.Close ()
	ReturnButton.OnPress (CloseStartOptions)
	ReturnButton.MakeEscape()

	OptionsWindow.Focus()
	GemRB.GetView ("STARTWIN").SetDisabled (True)

	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2005 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import GemRB
import GUICommon
import GUICommonWindows
import GUIClasses
import PortraitWindow
from GameCheck import MAX_PARTY_SIZE
from GUIDefines import *

MTASize = GS_SMALLDIALOG
smallID = -1
largeID = -1

def OnLoad():
	global smallID, largeID
	OptionsWindow = GemRB.LoadWindow(0, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)
	OptionsWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	OptionsWindow.AddAlias("OPTWIN")
	OptionsWindow.AddAlias("HIDE_CUT", 2)
	OptionsWindow.AddAlias("NOT_DLG", 1)
	GUICommonWindows.SetupMenuWindowControls (OptionsWindow, 1, None)

	# this also has the small MTA window
	OptionsWindow.AddAlias("MSGWIN")
	# compensate for the bogus scrollbar on the CHU BG
	smallMTA = OptionsWindow.GetControl(1)
	smallMTA.AddAlias("MsgSys", 0)
	smallMTA.AddAlias("MTA_SM", 0, True)
	smallMTA.SetFlags (IE_GUI_TEXTAREA_AUTOSCROLL|IE_GUI_TEXTAREA_HISTORY)
	smallMTA.SetColor(ColorRed, TA_COLOR_OPTIONS)
	smallMTA.SetColor(ColorWhite, TA_COLOR_HOVER)
	smallID = smallMTA.ID
	
	sbar = OptionsWindow.GetControl(2)
	sbar.SetResizeFlags(IE_GUI_VIEW_RESIZE_VERTICAL | IE_GUI_VIEW_RESIZE_RIGHT)

	# remove the cheat input textedit
	OptionsWindow.RemoveSubview(OptionsWindow.GetControl(3))

	ActionsWindow = PortraitWindow.OpenPortraitWindow(0, WINDOW_BOTTOM|WINDOW_HCENTER)
	ActionsWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	ActionsWindow.AddAlias("ACTWIN")
	ActionsWindow.AddAlias("HIDE_CUT", 1)
	ActionsWindow.AddAlias("NOT_DLG", 0)

	actframe = ActionsWindow.GetFrame()
	optframe = OptionsWindow.GetFrame()

	actframe['y'] -= optframe['h']
	ActionsWindow.SetFrame(actframe)

	DialogWindow = GemRB.LoadWindow(7, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)
	DialogWindow.SetFlags(WF_BORDERLESS|IE_GUI_VIEW_IGNORE_EVENTS|IE_GUI_VIEW_INVISIBLE, OP_OR)
	DialogWindow.AddAlias("MSGWINLG")
	DialogWindow.AddAlias("HIDE_CUT", 0)
	DialogWindow.AddAlias("NOT_DLG", 2)

	largeMTA = DialogWindow.GetControl(1)
	largeMTA.AddAlias("MTA_LG", 0, True)
	largeID = largeMTA.ID

	return

def UpdateControlStatus():
	global MTASize

	GSFlags = GemRB.GetGUIFlags()
	Expand = GSFlags&GS_DIALOGMASK
	InDialog = GSFlags&GS_DIALOG
	GSFlags = GSFlags-Expand

	largeMW = GemRB.GetView("MSGWINLG")

	#a dialogue is running, setting messagewindow size to maximum
	if InDialog:
		Expand = GS_LARGEDIALOG
		largeMW.SetVisible(True)
		largeMW.SetDisabled(False)
	else:
		largeMW.SetVisible(False)

	if MTASize == Expand:
		return

	MTASize = Expand

	smallMW = GemRB.GetView("MSGWIN")
	smallMTA = GemRB.GetView("MTA_SM", 0)
	smallFrame = smallMTA.GetFrame()

	largeMTA = GemRB.GetView("MTA_LG", 0)
	largeFrame = largeMTA.GetFrame()

	# swap the 2 TextAreas
	smallMTA = smallMW.RemoveSubview(smallMTA)
	largeMTA = largeMW.RemoveSubview(largeMTA)

	newLargeMTA = largeMW.AddSubview(smallMTA, None, largeID)
	newLargeMTA.SetFrame(largeFrame)
	newLargeMTA.AddAlias("MTA_LG", 0, True)

	newSmallMTA = smallMW.AddSubview(largeMTA, None, smallID)
	newSmallMTA.SetFrame(smallFrame)
	newSmallMTA.AddAlias("MTA_SM", 0, True)
	
	if InDialog:
		#gets PC currently talking
		pc = GemRB.GameGetSelectedPCSingle (1)
		if pc:
			Portrait = GemRB.GetPlayerPortrait(pc, 1)["Sprite"]
		else:
			Portrait = None
		Button = largeMW.GetControl(11)
		Button.SetState (IE_GUI_BUTTON_LOCKED)
		if not Portrait:
			Button.SetFlags(IE_GUI_BUTTON_NO_IMAGE, OP_SET)
		else:
			Button.SetPicture(Portrait, "NOPORTSM")
	else:
		newSmallMTA.ScrollTo(0, -9999) # scroll to the bottom
		
	return




# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation (GUICG 0)

import CharGenCommon
import CharGenGui

def init():
	print("init")
	# (name, control, text)
	# (name, script | setFn, commentFn, unsetFn, guard)
	stages = [
	("gender"	 , 0		, 11956		),
	("Show name, create player" , None    ,CharGenGui.getName, CharGenGui.unsetPlayer,  CharGenGui.setPlayer ),
	("setGender"	, "GUICG1"	, CharGenGui.getGender	,CharGenGui.unsetGender		, None		), #BG2: same
	("setPortrait"	, "GUICG12"	, None		,CharGenGui.unsetPortrait		, None		), #BG2: different setPicture, PortraitName and extra unused control (TextAreaControl)
	("race"		, 1		,11957		),
	("setRace"	, "GUICG8"	, CharGenGui.getRace	,CharGenGui.unsetRace 		, None		), #BG2: different resources
	("class"	, 2		,11959		),
	("setClass"	, "GUICG2"	, CharGenGui.getClass 	,CharGenGui.unsetClass		, None 		), #BG2: different resources,other setLogic, has levels/XP
	("setMultiClass", "GUICG10"	, None		,CharGenGui.unsetClass	, CharGenGui.guardMultiClass), #BG2: not same?
	("setSpecialist", "GUICG22"	, None		,CharGenGui.unsetClass	, CharGenGui.guardSpecialist), #BG2: not same
	("alignment"	, 3		,11958		),
	("setAlignment"	, "GUICG3"	, CharGenGui.getAlignment	,CharGenGui.unsetAlignment		, None		), #bg2: BG1 doesn't take kits into account
	("abilities"	, 4		,11960		),
	("setAbilities"	, "GUICG4"	, CharGenGui.getAbilities	,CharGenGui.unsetAbilities		, None		), #bg2: BG2 has extra strength, overpress (functionality + resources), kits
	("skill"	, 5		,17372		),
	("divine spells" , None , CharGenGui.getDivineSpells,CharGenGui.unsetDivineSpells,CharGenGui.setDivineSpells ),
	("hate race"	, "GUICG15"	, CharGenGui.getHatedRace	,CharGenGui.unsetHateRace		, CharGenGui.guardHateRace), #BG2: other listsize, button offset, BG2 has TopIndex
	("mage spells"	, "GUICG7"	, CharGenGui.getMageSpells,CharGenGui.unsetMageSpells		, CharGenGui.guardMageSpells),   #also sets priest in CharGen6, we do it at the end, other animations for buttons
	("setSkill"	, "GUICG6"	, CharGenGui.getSkills		,CharGenGui.unsetSkill		, CharGenGui.guardSkills	),
	("proficiencies", "GUICG9"	, CharGenGui.getProfi		,CharGenGui.unsetProfi		, None		),
	("appearance"	, 6		,11961		),
	("colors"	, "GUICG13"	, None		,CharGenGui.unsetColors	 	, None		), #bg2: other cycleids (SetBAM)
	("sounds"	, "GUICG19"	, None		,CharGenGui.unsetSounds		, None		), #BG2: other: first male sound, logic
	("name"		, 7 		,11963		),
	("setName"	 ,"GUICG5"	, None 		,CharGenGui.unsetName		, None 		), #BG2: same	
	("accept"	, 8		,11962		),
	("finish"	, CharGenGui.setAccept	, None 		, None			, None )]
	
	CharGenCommon.CharGenMaster = CharGenCommon.CharGen(stages,16575,CharGenGui.Imprt)

	return

def OnLoad():
	if(CharGenCommon.CharGenMaster == None):
		init()
	CharGenCommon.CharGenMaster.displayOverview()
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, import (GUICG24)
import GemRB
import CharGenCommon

#import from a character sheet
ImportWindow = 0
TextAreaControl = 0

def OnLoad():
	global ImportWindow, TextAreaControl

	ImportWindow = GemRB.LoadWindow(20, "GUICG")
	CharGenCommon.PositionCharGenWin(ImportWindow)

	TextAreaControl = ImportWindow.GetControl(4)
	TextAreaControl.SetText(10963)

	TextAreaControl = ImportWindow.GetControl(2)
	TextAreaControl.ListResources(CHR_EXPORTS)
 
	DoneButton = ImportWindow.GetControl(0)
	DoneButton.SetText(2610)
	DoneButton.SetDisabled(True)
	DoneButton.MakeDefault()

	CancelButton = ImportWindow.GetControl(1)
	CancelButton.SetText(15416)
	CancelButton.MakeEscape()

	DoneButton.OnPress (DonePress)
	CancelButton.OnPress (CancelPress)
	TextAreaControl.OnSelect (SelectPress)
	ImportWindow.Focus()
	return

def SelectPress():
	DoneButton = ImportWindow.GetControl(0)
	DoneButton.SetDisabled(False)
	return

def DonePress():
	FileName = TextAreaControl.QueryText()
	Slot = GemRB.GetVar("Slot")
	GemRB.CreatePlayer(FileName, Slot| 0x8000, 1, 11) # 11 = force bg2
	GemRB.SetToken ("CHARNAME", GemRB.GetPlayerName (Slot))
	if ImportWindow:
		ImportWindow.Close ()
	# the medium portrait isn't available, so we copy the original hack
	SmallPortrait = GemRB.GetPlayerPortrait (Slot, 1)["ResRef"]
	MediumPortrait = SmallPortrait[0:-1] + "M"
	GemRB.SetToken ("SmallPortrait", SmallPortrait)
	GemRB.SetToken ("LargePortrait", MediumPortrait)
	GemRB.SetNextScript("CharGen7")
	GemRB.SetVar ("ImportedChar", 1)
	return
	
def CancelPress():
	if ImportWindow:
		ImportWindow.Close ()
	GemRB.SetNextScript("CharGen")
	return 



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# LUSkillSelection.py - selection of thief skills

import GemRB
from GUIDefines import *
from ie_stats import *
import GameCheck
import GUICommon
import CommonTables
from CommonWindow import AddScrollbarProxy

#constants
LUSKILLS_TYPE_LEVELUP = 1
LUSKILLS_TYPE_CHARGEN = 2
LUSKILLS_TYPE_DUALCLASS = 4
LUSKILLS_MAX = 250

#refs to the script calling this
SkillsWindow = 0
SkillsCallback = 0

#offsets to the various parts of all 3 windows
SkillsOffsetPress = 0
SkillsOffsetName = 0
SkillsOffsetPoints = 0
SkillsOffsetSum = 0
SkillsNumButtons = 0

#The number of steps between each control ID for the skill label and points of each skill
#Set to 2 for PST, as they are 'interleaved' ie label/amount/label/amount
SkillsLabelIncrement = 1

#internal variables
SkillsIndices = []
SkillPointsLeft = 0
SkillsTable = 0
SkillsOldPos = 0
SkillsClickCount = 0
SkillsOldDirection = 0
SkillsTextArea = 0
SkillsKitName = 0
SkillsAssignable = 0

#WARNING: This WILL NOT show the window, only access it. To see the return, call GemRB.GetVar ("SkillPointsLeft").
# If nothing can be assigned, it will return 0 prior to accessing any of the window methods.
def SetupSkillsWindow (pc, skilltype, window, callback, level1=[0,0,0], level2=[1,1,1], classid=0, scroll=True):
	global SkillsWindow, SkillsCallback, SkillsOffsetPress, SkillsOffsetName
	global SkillsOffsetPoints, SkillsOffsetSum, SkillsIndices, SkillPointsLeft
	global SkillsTable, SkillsOldPos, SkillsClickCount, SkillsOldDirection, SkillsNumButtons
	global SkillsTextArea, SkillsKitName, SkillsAssignable, SkillsLabelIncrement

	#reset some basic values
	SkillLeftPress = SkillIncreasePress
	SkillRightPress = SkillDecreasePress
	SkillsWindow = window
	SkillsCallback = callback
	SkillsOldPos = 0
	SkillsClickCount = 0
	SkillsOldDirection = 0
	SkillsAssignable = 0
	SkillsTable = GemRB.LoadTable ("skills")
	SkillPointsLeft = 0
	GemRB.SetVar ("SkillPointsLeft", 0)
	SkillsNullify ()
	ScrollBar = None

	#make sure we're within ranges
	if not window or not callback or len(level1)!=len(level2):
		return

	#setup the offsets
	if skilltype == LUSKILLS_TYPE_LEVELUP and GameCheck.IsBG2OrEE ():
		SkillsOffsetPress = 120
		SkillsOffsetButton1 = 17
		SkillsOffsetSum = 37
		SkillsOffsetName = 32
		SkillsOffsetPoints = 43
		SkillsNumButtons = 4
		SkillsTextArea = SkillsWindow.GetControl (110)
		ScrollBar = SkillsWindow.GetControl (109)
	elif skilltype == LUSKILLS_TYPE_LEVELUP and GameCheck.IsPST():
		SkillsOffsetPress = -1
		SkillsOffsetButton1 = 16
		SkillsOffsetSum = 6
		SkillsOffsetName = 7
		SkillsOffsetPoints = 8
		SkillsNumButtons = 4
		SkillsLabelIncrement = 2
		#The order of the buttons is the opposite of the other games
		SkillLeftPress = SkillDecreasePress
		SkillRightPress = SkillIncreasePress
		#There is actually no hint text to describe the skills, so this is a dummy
		SkillsWindow.CreateTextArea (45, 1, 1, 1, 1, "FONTDLG")
		SkillsTextArea =  SkillsWindow.GetControl (45)
	elif skilltype == LUSKILLS_TYPE_LEVELUP:
		SkillsOffsetPress = -1
		SkillsOffsetButton1 = 17
		SkillsOffsetSum = 37
		SkillsOffsetName = 32
		SkillsOffsetPoints = 43
		SkillsNumButtons = 4
		SkillsTextArea = SkillsWindow.GetControl (42)
		if (scroll):
			ScrollBar = SkillsWindow.GetControl (109)
	elif skilltype == LUSKILLS_TYPE_DUALCLASS:
		SkillsOffsetPress = 5
		SkillsOffsetButton1 = 14
		SkillsOffsetSum = 8
		SkillsOffsetName = 0
		SkillsOffsetPoints = 9
		SkillsNumButtons = 4
		SkillsTextArea = SkillsWindow.GetControl (22)
		SkillsTextArea.SetText(17248)
		if (scroll):
			ScrollBar = SkillsWindow.GetControl (26)
			SkillsWindow.SetEventProxy(ScrollBar)
	elif skilltype == LUSKILLS_TYPE_CHARGEN:
		SkillsOffsetPress = 21
		SkillsOffsetButton1 = 11
		SkillsOffsetSum = 5
		SkillsOffsetName = 6
		SkillsOffsetPoints = 1
		SkillsNumButtons = 4
		SkillsTextArea = SkillsWindow.GetControl (19)
		SkillsTextArea.SetText(17248)
		if (scroll):
			ScrollBar = SkillsWindow.GetControl (26)
			SkillsWindow.SetEventProxy(ScrollBar)
	else:
		return

	# yet another "what were they thinking?!"
	if GameCheck.IsBG2EE () and skilltype == LUSKILLS_TYPE_DUALCLASS:
		# remap 9,16,18,20 to 14,16,18,20 AND 10,17,19,21 to 15,17,19,21
		# potentially also broken in chargen
		SkillsWindow.AliasControls ({'PLUSBTN' + str(x[0]) : x[1] for x in enumerate([9, 16, 18, 20])})
		SkillsWindow.AliasControls ({'MINUSBTN' + str(x[0]) : x[1] for x in enumerate([10, 17, 19, 21])})
	else:
		SkillsWindow.AliasControls ({'PLUSBTN' + str(x) : x * 2 + SkillsOffsetButton1 for x in range(4)})
		SkillsWindow.AliasControls ({'MINUSBTN' + str(x) : x * 2 + SkillsOffsetButton1 + 1 for x in range(4)})

	if ScrollBar:
		ScrollBar.SetVarAssoc ("SkillsTopIndex", 0, 0, 0)
	#get our class id and name
	IsDual = GUICommon.IsDualClassed (pc, 1)
	IsMulti = GUICommon.IsMultiClassed (pc, 1)
	if classid: #used when dual-classing
		Class = classid
	elif IsDual[0] == 3:
		Class = CommonTables.KitList.GetValue (IsDual[2], 7)
	elif IsDual[0]: #only care about the current class
		Class = GUICommon.GetClassRowName(IsDual[2], "index")
		Class = CommonTables.Classes.GetValue (Class, "ID")
	else:
		Class = GemRB.GetPlayerStat (pc, IE_CLASS)
	ClassName = GUICommon.GetClassRowName(Class, "class")

	#get the number of classes
	if IsMulti[0]>1:
		NumClasses = IsMulti[0]
	else:
		NumClasses = 1
	if NumClasses > len (level2):
		return

	#figure out the kitname if we need it
	#protect against kitted multiclasses
	Kit = GUICommon.GetKitIndex (pc)
	if not Kit or skilltype == LUSKILLS_TYPE_DUALCLASS or IsDual[0] in [1, 2] or IsMulti[0]>1:
		SkillsKitName = ClassName
	else:
		SkillsKitName = CommonTables.KitList.GetValue (Kit, 0, GTV_STR)

	# also treat most mod-introduced kits as kitless for skills.2da
	# lookups - unless they add the required columns
	if SkillsTable.GetValue ("OPEN_LOCKS", SkillsKitName) == -1:
		SkillsKitName = ClassName

	#figure out the correct skills table
	SkillIndex = -1
	for i in range (NumClasses):
		TmpClass = Class
		if NumClasses > 1:
			TmpClass = IsMulti[i+1]
		TmpClass = GUICommon.GetClassRowName (TmpClass, "class")
		if (CommonTables.ClassSkills.GetValue (TmpClass, "THIEFSKILL", GTV_STR) != "*"):
			SkillIndex = i
			break

	#see if we got a thief (or monk)
	SkillsIndices = []
	if SkillIndex >= 0:
		#SkillsKitName should be fine as all multis are in classes.2da
		#also allows for thief kits
		SkillsAssignable = 1
		for i in range(SkillsTable.GetRowCount()-2):
			# -2/+2 to compensate for the special first_level and rate rows
			SkillName = SkillsTable.GetRowName (i+2)
			if SkillsTable.GetValue (SkillName, SkillsKitName) is not None:
				SkillsIndices.append(i)

		LevelDiff = []
		for i in range (NumClasses):
			LevelDiff.append (level2[i]-level1[i])
		if level1[SkillIndex] == 0:
			SkillPointsLeft = SkillsTable.GetValue ("FIRST_LEVEL", SkillsKitName, GTV_INT)
			LevelDiff[SkillIndex] -= 1
		SkillPointsLeft += LevelDiff[SkillIndex] * SkillsTable.GetValue("RATE", SkillsKitName, GTV_INT)
		TotalSkillsAssignable = 0

		if SkillPointsLeft < 0:
			#really don't have an entry
			SkillPointsLeft = 0
		else:
			#get the skill values
			for i in range(SkillsTable.GetRowCount()-2):
				# -2/+2 to compensate for the special first_level and rate rows
				SkillName = SkillsTable.GetRowName (i+2)
				SkillID = SkillsTable.GetValue (SkillName, "ID")
				SkillValue = GemRB.GetPlayerStat (pc, SkillID)
				BaseSkillValue = GemRB.GetPlayerStat (pc, SkillID, 1)
				GemRB.SetVar("Skill "+str(i), SkillValue)
				GemRB.SetVar("SkillBase "+str(i), SkillValue)
				# display the modified stat to avoid confusion (account for dex, race and effect boni)
				GemRB.SetVar("SkillDisplayMod "+str(i), SkillValue-BaseSkillValue)
				TotalSkillsAssignable += LUSKILLS_MAX-SkillValue

		#protect against having more skills than we can assign
		if SkillPointsLeft > TotalSkillsAssignable:
			SkillPointsLeft = TotalSkillsAssignable
		GemRB.SetVar ("SkillPointsLeft", SkillPointsLeft)
	else: 
		#get ranger and bard skills
		SpecialSkillsMap = []
		for i in range(NumClasses):
			if IsMulti[0]>1:
				classname = IsMulti[i+1]
			else:
				classname = Class
			classname = GUICommon.GetClassRowName (classname, "class")
			for table in "RANGERSKILL", "BARDSKILL":
				SpecialSkillsTable = CommonTables.ClassSkills.GetValue (classname, table)
				if SpecialSkillsTable != "*":
					SpecialSkillsMap.append((SpecialSkillsTable, i))
					break
		for skills in SpecialSkillsMap:
			SpecialSkillsTable = GemRB.LoadTable (skills[0])
			for skill in range(SpecialSkillsTable.GetColumnCount ()):
				skillname = SpecialSkillsTable.GetColumnName (skill)
				value = SpecialSkillsTable.GetValue (str(level2[skills[1]]), skillname)
				skillindex = SkillsTable.GetRowIndex (skillname) - 2
				GemRB.SetVar ("Skill " + str(skillindex), value)
				SkillsIndices.append(skillindex)

	#we didn't find anything, so don't continue (will show as a return of 0)
	#or don't display if we aren't leveling and have a bard/ranger
	if not len (SkillsIndices) or (not SkillPointsLeft and skilltype != LUSKILLS_TYPE_LEVELUP):
		SkillSumLabel = SkillsWindow.GetControl(0x10000000+SkillsOffsetSum)
		SkillSumLabel.SetText("")
		return

	#skills scrollbar
	if len(SkillsIndices) > SkillsNumButtons:
		ScrollBar.OnChange (lambda: SkillsRedraw())
		#decrease it with the number of controls on screen (list size) and two unrelated rows
		maxvalue = SkillsTable.GetRowCount() - SkillsNumButtons - 2
		ScrollBar.SetVarAssoc ("SkillsTopIndex", 0, 0, maxvalue)
		Button = SkillsWindow.GetControl(SkillsOffsetPress)
		AddScrollbarProxy(SkillsWindow, ScrollBar, Button)
	else:
		GemRB.SetVar ("SkillsTopIndex", 0)

	#setup all the visible buttons
	for i in range(len(SkillsIndices)):
		if i == SkillsNumButtons:
			break
		if SkillsOffsetPress != -1:
			Button = SkillsWindow.GetControl(i+SkillsOffsetPress)
			Button.SetVarAssoc("Skill",SkillsIndices[i])
			Button.OnPress (SkillJustPress)

		Button = SkillsWindow.GetControlAlias ("PLUSBTN" + str(i))
		Button.SetVarAssoc("Skill",SkillsIndices[i])
		Button.OnPress (SkillLeftPress)
		Button.SetActionInterval(20);

		Button =  SkillsWindow.GetControlAlias ("MINUSBTN" + str(i))
		Button.SetVarAssoc("Skill",SkillsIndices[i])
		Button.OnPress (SkillRightPress)
		Button.SetActionInterval(20);
	
	SkillsRedraw ()
	return

def SkillsRedraw (direction=0):
	global SkillsOldDirection, SkillsClickCount, SkillsLabelIncrement

	#update how many skill points are left and call the callback function
	SkillSumLabel = SkillsWindow.GetControl(0x10000000+SkillsOffsetSum)
	SkillSumLabel.SetText(str(SkillPointsLeft))

	for i in range (SkillsNumButtons):
		if len(SkillsIndices) <= i:
			SkillsHide (i)
			continue

		#show the current skills name
		Pos = SkillsIndices[GemRB.GetVar ("SkillsTopIndex") + i]
		SkillName = SkillsTable.GetValue (SkillsTable.GetRowName (Pos+2), "CAP_REF")
		Label = SkillsWindow.GetControl (0x10000000+SkillsOffsetName+(i*SkillsLabelIncrement))
		Label.SetText (SkillName)

		#enable/disable the button if we can(not) get the skills
		SkillName = SkillsTable.GetRowName (Pos+2)
		Ok = SkillsTable.GetValue (SkillName, SkillsKitName) and SkillsAssignable
		Button1 = SkillsWindow.GetControlAlias ("PLUSBTN" + str(i))
		Button2 = SkillsWindow.GetControlAlias ("MINUSBTN" + str(i))
		if not Ok:
			Button1.SetState(IE_GUI_BUTTON_DISABLED)
			Button2.SetState(IE_GUI_BUTTON_DISABLED)
			Button1.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_OR)
			Button2.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_OR)
		else:
			Button1.SetState(IE_GUI_BUTTON_ENABLED)
			Button2.SetState(IE_GUI_BUTTON_ENABLED)
			Button1.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_NAND)
			Button2.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_NAND)
	
		#show how many points are allocated to this skill		
		Label = SkillsWindow.GetControl(0x10000000+SkillsOffsetPoints+(i*SkillsLabelIncrement))
		ActPoint = GemRB.GetVar("Skill "+str(Pos) )
		Label.SetText(str(ActPoint))

	#setup doublespeed
	if SkillsOldDirection == direction:
		SkillsClickCount = SkillsClickCount + 1
		return

	SkillsOldDirection = direction
	SkillsClickCount = 0
	return

def SkillJustPress (btn):
	Pos = btn.Value + GemRB.GetVar ("SkillsTopIndex")
	SkillsTextArea.SetText (SkillsTable.GetValue (SkillsTable.GetRowName (Pos+2), "DESC_REF"))
	return

def SkillDecreasePress (btn):
	global SkillPointsLeft, SkillsClickCount, SkillsOldPos

	Pos = btn.Value + GemRB.GetVar ("SkillsTopIndex")
	SkillsTextArea.SetText (SkillsTable.GetValue (SkillsTable.GetRowName (Pos+2), "DESC_REF"))
	ActPoint = GemRB.GetVar("Skill "+str(Pos) )
	BasePoint = GemRB.GetVar("SkillBase "+str(Pos) )
	if ActPoint <= 0 or ActPoint <= BasePoint:
		return
	GemRB.SetVar("Skill "+str(Pos),ActPoint-1)
	SkillPointsLeft = SkillPointsLeft + 1
	if SkillsOldPos != Pos:
		SkillsOldPos = Pos
		SkillsClickCount = 0

	GemRB.SetVar ("SkillPointsLeft", SkillPointsLeft)
	SkillsRedraw(2)
	SkillsCallback ()
	return

def SkillIncreasePress (btn):
	global SkillPointsLeft, SkillsClickCount, SkillsOldPos

	Pos = btn.Value + GemRB.GetVar ("SkillsTopIndex")
	SkillsTextArea.SetText (SkillsTable.GetValue (SkillsTable.GetRowName (Pos+2), "DESC_REF"))
	if SkillPointsLeft == 0:
		return
	ActPoint = GemRB.GetVar("Skill "+str(Pos) )
	if ActPoint >= LUSKILLS_MAX:
		return
	GemRB.SetVar("Skill "+str(Pos), ActPoint+1)
	SkillPointsLeft = SkillPointsLeft - 1
	if SkillsOldPos != Pos:
		SkillsOldPos = Pos
		SkillsClickCount = 0

	GemRB.SetVar ("SkillPointsLeft", SkillPointsLeft)
	SkillsRedraw(1)
	SkillsCallback ()
	return

# saves all the skills
def SkillsSave (pc):
	global SkillsTable
	if not SkillsTable:
		SkillsTable = GemRB.LoadTable ("skills")

	for i in range(SkillsTable.GetRowCount() - 2):
		SkillName = SkillsTable.GetRowName (i+2)
		SkillID = SkillsTable.GetValue (SkillName, "ID")
		SkillValue = (GemRB.GetVar ("Skill " + str(i)) or 0) - (GemRB.GetVar("SkillDisplayMod " + str(i)) or 0)
		if SkillValue > 0:
			GemRB.SetPlayerStat (pc, SkillID, SkillValue)

def SkillsNullify (pc = None):
	global SkillsTable
	if not SkillsTable:
		SkillsTable = GemRB.LoadTable ("skills")

	for i in range(SkillsTable.GetRowCount()-2):
		GemRB.SetVar ("Skill "+str(i), 0)
		GemRB.SetVar ("SkillBase "+str(i), 0)
		if pc:
			SkillName = SkillsTable.GetRowName (i+2)
			SkillID = SkillsTable.GetValue (SkillName, "ID")
			GemRB.SetPlayerStat (pc, SkillID, 0)

def SkillsHide (i):
	Label = SkillsWindow.GetControl (0x10000000+SkillsOffsetName+i)
	Label.SetText ("")
	Button1 = SkillsWindow.GetControlAlias ("PLUSBTN" + str(i))
	Button1.SetState(IE_GUI_BUTTON_DISABLED)
	Button1.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_OR)
	Button2 = SkillsWindow.GetControlAlias ("MINUSBTN" + str(i))
	Button2.SetState(IE_GUI_BUTTON_DISABLED)
	Button2.SetFlags(IE_GUI_BUTTON_NO_IMAGE,OP_OR)
	Label = SkillsWindow.GetControl(0x10000000+SkillsOffsetPoints+i)
	Label.SetText("")



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, import (GUICG24)
import GemRB

ImportWindow = 0

def OnLoad():
	global ImportWindow

	ImportWindow = GemRB.LoadWindow(24, "GUICG")
	ImportWindow.SetFlags (WF_ALPHA_CHANNEL, OP_OR)

	TextAreaControl = ImportWindow.GetControl(0)
	TextAreaControl.SetText(53605)

	FileButton = ImportWindow.GetControl(1)
	FileButton.SetText(53604)

	SavedGameButton = ImportWindow.GetControl(2)
	SavedGameButton.SetText(53602)

	CancelButton = ImportWindow.GetControl(3)
	CancelButton.SetText(13727)
	CancelButton.MakeEscape()

	FileButton.OnPress (FilePress)
	SavedGameButton.OnPress (GamePress)
	CancelButton.OnPress (CancelPress)
	ImportWindow.ShowModal(MODAL_SHADOW_GRAY)
	return

def FilePress():
	if ImportWindow:
		ImportWindow.Close ()
	GemRB.SetNextScript("ImportFile")
	return
	
def GamePress():
	if ImportWindow:
		ImportWindow.Close ()
	GemRB.SetNextScript("ImportGame")
	return

def CancelPress():
	if ImportWindow:
		ImportWindow.Close ()
	GemRB.SetNextScript("CharGen")
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUIMA.py - scripts to control map windows from GUIMA and GUIWMAP winpacks

###################################################

import GemRB
import GUICommon
import GUICommonWindows
import GUIMACommon
from GUIDefines import *

WorldMapControl = None

###################################################
def InitMapWindow (Window):
	Window.AddAlias("WIN_MAP")

	# World Map
	Button = Window.GetControl (0)
	Button.SetText (20429)
	Button.OnPress (lambda: OpenTravelWindow(None))

	# Add Note
	Button = Window.GetControl (1)
	Button.SetText (4182)
	Button.SetVarAssoc ("ShowMapNotes", IE_GUI_MAP_SET_NOTE)

	# Note text
	Edit = Window.GetControl (4)
	Edit.OnDone (NoteChanged)
	Edit.SetFlags(IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)

	# Map Control
	# ronote and usernote are the pins for the notes
	Map = Window.ReplaceSubview(3, IE_GUI_MAP, Edit)
	Map.AddAlias("MAP")

	Map.SetVarAssoc ("ShowMapNotes", IE_GUI_MAP_VIEW_NOTES)
	Map.SetStatus (IE_GUI_MAP_VIEW_NOTES)

	Map.OnPress (SetMapNote)
	Map.SetAction(Window.Close, IE_ACT_MOUSE_PRESS, GEM_MB_ACTION, 0, 2)

	MapTable = GemRB.LoadTable( "MAPNAME" )
	MapName = MapTable.GetValue (GemRB.GetCurrentArea (), "STRING")
	
	Label = Window.GetControl (0x10000005)
	Label.SetText (MapName)
	#Label.SetColor (255, 0, 0)
	# 2 - map name?
	# 3 - map bitmap?
	# 4 - ???

	# these overlap the map. lets move them to be children of the map
	Label = Window.ReparentSubview(Label, Map)
	Edit = Window.ReparentSubview(Edit, Map)

	# Done
	Button = Window.GetControl (5)
	Button.SetText (1403)
	Button.OnPress (Window.Close)
	Button.MakeEscape()

	return

def OpenTravelWindow (Travel):
	global WorldMapControl

	Window = GemRB.LoadWindow (0, "GUIWMAP")
	Window.AddAlias ("WIN_PSTWMAP")

	color = {'r' : 30, 'g' : 8, 'b' : 0, 'a' : 0xff}
	WorldMapControl = WMap = Window.ReplaceSubview (4, IE_GUI_WORLDMAP, "FONTDLG", "WMPTY", color, color, color)
	WMap.SetVarAssoc ("Travel", Travel)
	#center on current area
	WMap.Scroll (0,0)
	WMap.Focus()
	if Travel is not None:
		WMap.OnPress (GUIMACommon.MoveToNewArea)

	# Done
	Button = Window.GetControl (0)
	Button.SetText (1403)
	Button.OnPress (Window.Close)

ToggleMapWindow = GUICommonWindows.CreateTopWinLoader(3, "GUIMA", GUICommonWindows.ToggleWindow, InitMapWindow)
OpenMapWindow = GUICommonWindows.CreateTopWinLoader(3, "GUIMA", GUICommonWindows.OpenWindowOnce, InitMapWindow)

def NoteChanged (Edit):
	Edit.SetFlags(IE_GUI_VIEW_IGNORE_EVENTS, OP_OR)
	Text = Edit.QueryText ()
	Edit.SetText ("")
	Edit.SetBackground ("MAMOSED1")
	PosX = GemRB.GetVar("MapControlX")
	PosY = GemRB.GetVar("MapControlY")
	GemRB.SetMapnote (PosX, PosY, 1, Text)

	Map = GemRB.GetView("MAP")
	Map.SetVarAssoc ("ShowMapNotes", IE_GUI_MAP_VIEW_NOTES)
	Map.SetStatus (IE_GUI_MAP_VIEW_NOTES)

	return
	
def SetMapNote (Map):
	if GemRB.GetVar ("ShowMapNotes") != IE_GUI_MAP_SET_NOTE:
		return

	MapWindow = GemRB.GetView("WIN_MAP")

	Edit = MapWindow.GetControl (4)
	Edit.SetFlags (IE_GUI_VIEW_IGNORE_EVENTS, OP_NAND)
	Edit.Focus()

	return

###################################################
# End of file GUIMA.py




# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
#character generation, import (GUICG20)
import GemRB
from GUIDefines import *
import GUICommon
import CharGenCommon

#import from a character sheet
ImportWindow = 0
TextAreaControl = 0

def OnLoad():
	global ImportWindow, TextAreaControl

	ImportWindow = GemRB.LoadWindow(20, "GUICG")

	TextAreaControl = ImportWindow.GetControl(4)
	TextAreaControl.SetText(10963)

	TextAreaControl = ImportWindow.GetControl(2)
	TextAreaControl.ListResources(CHR_EXPORTS)

	DoneButton = ImportWindow.GetControl(0)
	DoneButton.SetText (11973)

	DoneButton.SetState(IE_GUI_BUTTON_DISABLED)

	CancelButton = ImportWindow.GetControl(1)
	CancelButton.SetText (13727)

	DoneButton.OnPress (DonePress)
	CancelButton.OnPress (CancelPress)
	TextAreaControl.OnSelect (SelectPress)
	ImportWindow.ShowModal(MODAL_SHADOW_GRAY)
	return

def SelectPress():
	DoneButton = ImportWindow.GetControl(0)
	DoneButton.SetState(IE_GUI_BUTTON_ENABLED)
	return

def DonePress():
	ImportWindow.Close()
	FileName = TextAreaControl.QueryText()
	Slot = GemRB.GetVar("Slot")
	GemRB.CreatePlayer(FileName, Slot| 0x8000, 1)

	GemRB.SetToken ("CHARNAME", GemRB.GetPlayerName (Slot))
	GemRB.SetToken ("SmallPortrait", GemRB.GetPlayerPortrait (Slot, 1)["ResRef"])
	GemRB.SetToken ("LargePortrait", GemRB.GetPlayerPortrait (Slot, 0)["ResRef"])

	GemRB.SetVar ("ImportedChar", 1)
	CharGenCommon.jumpTo("appearance")
	return

def CancelPress():
	ImportWindow.Close()
	GemRB.SetNextScript(GemRB.GetToken("NextScript"))
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUILOAD.py - Load window

###################################################

import GemRB
import LoadScreen
from GameCheck import MAX_PARTY_SIZE
from GUIDefines import *

LoadWindow = 0
TextAreaControl = 0
Games = ()
ScrollBar = 0

def OnLoad ():
	global LoadWindow, TextAreaControl, Games, ScrollBar

	GemRB.SetToken ("SaveDir", "mpsave") # iwd2 is always using 'mpsave'
	LoadWindow = GemRB.LoadWindow (0, "GUILOAD")

	CancelButton=LoadWindow.GetControl (22)
	CancelButton.SetText (13727)
	CancelButton.OnPress (LoadWindow.Close)
	CancelButton.MakeEscape()

	for i in range (5):
		Button = LoadWindow.GetControl (55+i)
		Button.SetText (15590)
		Button.OnPress (LoadGamePress)
		Button.SetState (IE_GUI_BUTTON_DISABLED)
		Button.SetValue (i)

		Button = LoadWindow.GetControl (60+i)
		Button.SetText (13957)
		Button.OnPress (DeleteGamePress)
		Button.SetState (IE_GUI_BUTTON_DISABLED)
		Button.SetValue (i)

		#area previews
		Button = LoadWindow.GetControl (1+i)
		Button.SetState (IE_GUI_BUTTON_LOCKED)
		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE|IE_GUI_BUTTON_PICTURE,OP_SET)

		#PC portraits
		for j in range (min(6, MAX_PARTY_SIZE)):
			Button = LoadWindow.GetControl (25 + i*min(6, MAX_PARTY_SIZE) + j)
			Button.SetState (IE_GUI_BUTTON_LOCKED)
			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE|IE_GUI_BUTTON_PICTURE,OP_SET)
			Button.SetSize (21, 21)

	ScrollBar=LoadWindow.GetControl (23)
	ScrollBar.OnChange (ScrollBarPress)
	Games=GemRB.GetSaveGames()
	TopIndex = max (0, len(Games) - 5)

	ScrollBar.SetVarAssoc ("TopIndex", TopIndex, 0, TopIndex)
	ScrollBarPress ()

	LoadWindow.SetEventProxy(ScrollBar)
	LoadWindow.Focus()
	return

def ScrollBarPress ():
	#draw load game portraits
	Pos = GemRB.GetVar ("TopIndex") or 0
	for i in range (5):
		ActPos = Pos + i

		Button1 = LoadWindow.GetControl (55+i)
		Button2 = LoadWindow.GetControl (60+i)
		ScreenShotButton = LoadWindow.GetControl (1 + i)
		if ActPos<len(Games):
			Button1.SetState (IE_GUI_BUTTON_ENABLED)
			Button2.SetState (IE_GUI_BUTTON_ENABLED)
			ScreenShotButton.SetPicture (Games[ActPos].GetPreview())
			Slotname = Games[ActPos].GetName()
			GameDate = Games[ActPos].GetGameDate()
			SaveDate = Games[ActPos].GetDate()
		else:
			Button1.SetState (IE_GUI_BUTTON_DISABLED)
			Button2.SetState (IE_GUI_BUTTON_DISABLED)
			ScreenShotButton.SetPicture (None)
			Slotname = ""
			GameDate = ""
			SaveDate = ""

		Label = LoadWindow.GetControl (0x10000005+i)
		Label.SetText (Slotname)

		Label = LoadWindow.GetControl (0x1000000a+i)
		Label.SetText (GameDate)

		Label = LoadWindow.GetControl (0x1000000f+i)
		Label.SetText (SaveDate)

		for j in range (min(6, MAX_PARTY_SIZE)):
			Button=LoadWindow.GetControl (25 + i*min(6, MAX_PARTY_SIZE) + j)
			if ActPos<len(Games):
				Button.SetPicture (Games[ActPos].GetPortrait(j))
			else:
				Button.SetPicture (None)
	return

def LoadGamePress (btn):
	if LoadWindow:
		LoadWindow.Close ()
	Pos = GemRB.GetVar ("TopIndex") + btn.Value
	LoadScreen.StartLoadScreen()
	GemRB.LoadGame(Games[Pos]) #loads and enters savegame
	GemRB.EnterGame ()
	return

def DeleteGameConfirm (btn):
	global Games

	TopIndex = GemRB.GetVar ("TopIndex")
	Pos = TopIndex + btn.Value
	GemRB.DeleteSaveGame(Games[Pos])
	del Games[Pos]
	if TopIndex > 0:
		TopIndex = TopIndex - 1
	ScrollBar.SetVarAssoc ("TopIndex", TopIndex, 0, max (0, len(Games) - 5))
	ScrollBarPress ()
	if ConfirmWindow:
		ConfirmWindow.Close ()
	LoadWindow.Focus()
	return

def DeleteGameCancel ():
	if ConfirmWindow:
		ConfirmWindow.Close ()
	LoadWindow.Focus()
	return

def DeleteGamePress (btn):
	global ConfirmWindow

	ConfirmWindow=GemRB.LoadWindow (1, "GUILOAD")
	ConfirmWindow.SetFlags (WF_ALPHA_CHANNEL, OP_OR)

	Text=ConfirmWindow.GetControl (0)
	Text.SetText (15305)
	DeleteButton=ConfirmWindow.GetControl (1)
	DeleteButton.SetText (13957)
	DeleteButton.OnPress (DeleteGameConfirm)
	DeleteButton.SetValue (btn.Value)
	CancelButton=ConfirmWindow.GetControl (2)
	CancelButton.SetText (13727)
	CancelButton.OnPress (DeleteGameCancel)
	CancelButton.MakeEscape()

	ConfirmWindow.ShowModal (MODAL_SHADOW_GRAY)
	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2014 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#
# IDLUCommon.py - common functions related to leveling up in iwd2,
#                 that are incompatible with the other games

import GemRB
import CommonTables
import GUICommon
import Spellbook
from ie_stats import *
from ie_feats import *
from ie_restype import RES_SPL
from ie_spells import LS_MEMO
from GUIDefines import *

FeatSPLs = []

# barbarian, bard, cleric, druid, fighter, monk, paladin, ranger, rogue, sorcerer, wizard
# same order as in classes.2da / class IDs
Levels = [ IE_LEVELBARBARIAN, IE_LEVELBARD, IE_LEVELCLERIC, IE_LEVELDRUID, \
	IE_LEVEL, IE_LEVELMONK, IE_LEVELPALADIN, IE_LEVELRANGER, IE_LEVEL3, \
	IE_LEVELSORCERER, IE_LEVEL2 ]

def AddResistances(MyChar, rowname, table):
	resistances = GemRB.LoadTable (table)
	# add it to dmginfo.2da if we ever need it elsewhere
	titles = { IE_RESISTFIRE:"FIRE", IE_RESISTCOLD:"COLD", IE_RESISTELECTRICITY:"ELEC", \
		IE_RESISTACID:"ACID", IE_RESISTMAGIC:"SPELL", IE_RESISTMAGICFIRE:"MAGIC_FIRE", \
		IE_RESISTMAGICCOLD:"MAGIC_COLD", IE_RESISTSLASHING:"SLASHING", \
		IE_RESISTCRUSHING:"BLUDGEONING", IE_RESISTPIERCING:"PIERCING", IE_RESISTMISSILE:"MISSILE" }

	for resistance in titles:
		base = GemRB.GetPlayerStat (MyChar, resistance, 1)
		extra = resistances.GetValue (rowname, titles[resistance])
		GemRB.SetPlayerStat (MyChar, resistance, base+extra)
	return

# returns the race or subrace
def GetRace (pc):
	Race = GemRB.GetPlayerStat (pc, IE_RACE)
	Subrace = GemRB.GetPlayerStat (pc, IE_SUBRACE)
	if Subrace:
		Race = Race<<16 | Subrace
	return CommonTables.Races.FindValue (3, Race)

def SetupSavingThrows (pc, Class, Chargen=False):
	"""Updates an actors saving throws based upon level.

	Class should contain the class to consider for saving throw level-up.
	If Chargen is true, it will also add any racial boni."""

	# check if there are any racial bonuses to saves
	# they are constant in iwd2, regardless of level
	if Chargen:
		RaceSaveTable = GemRB.LoadTable ("SAVERACE")
		RaceName = CommonTables.Races.GetRowName (GetRace (pc))
		Race = RaceSaveTable.GetRowIndex (RaceName)
		# fortitude, reflex, will: colnames match stat order
		for i in range(3):
			RacialBonus = RaceSaveTable.GetValue (Race, i)
			GemRB.SetPlayerStat (pc, IE_SAVEFORTITUDE+i, RacialBonus)

	# class-based saving throw progression
	# get the actual xp level of the passed class
	LevelDiff = GemRB.GetVar ("LevelDiff") or 0 # only set during level-up
	Level = GemRB.GetPlayerStat(pc, Levels[Class-1]) + LevelDiff
	Level -= 1 # we'll use it as row index only

	RowName = GUICommon.GetClassRowName (Class, "class")
	SaveName = CommonTables.Classes.GetValue (RowName, "SAVE", GTV_STR)
	ClassSaveTable = GemRB.LoadTable (SaveName)
	if Level+1 > ClassSaveTable.GetRowCount():
		print("SetupSavingThrows: too high level, ignoring!")
		return

	# save the saves
	# fortitude, reflex, will: colnames match stat order
	# the tables contain absolute values, so there's no need to compute a diff
	# HOWEVER, permanent effects' effects would get obliterated if we didn't
	for i in range(3):
		Base = GemRB.GetPlayerStat (pc, IE_SAVEFORTITUDE+i, 1)
		OldClassBonus = ClassSaveTable.GetValue (Level-LevelDiff-1, i) # default is 0 if we underflow
		ClassBonus = ClassSaveTable.GetValue (Level, i)
		GemRB.SetPlayerStat (pc, IE_SAVEFORTITUDE+i, Base + ClassBonus-OldClassBonus)

def LearnAnySpells (pc, BaseClassName, chargen=1):
	MageTable = CommonTables.ClassSkills.GetValue (BaseClassName, "MAGESPELL", GTV_STR)
	ClericTable = CommonTables.ClassSkills.GetValue (BaseClassName, "CLERICSPELL", GTV_STR)

	# sorcerer types need this hack to not get too many spells
	# for them the castable and known count progress differently
	if MageTable == "MXSPLSOR":
		MageTable = "SPLSRCKN"
	elif MageTable == "MXSPLBRD":
		MageTable = "SPLBRDKN"

	# this is the current caster level to update to (for SetupSpellLevels)
	level = 1
	if not chargen:
		# LevelDiff + whatever caster level of this type we already had
		level = GemRB.GetVar ("LevelDiff") or 0
		classIndex = CommonTables.Classes.GetRowIndex (BaseClassName)
		level += GemRB.GetPlayerStat (pc, Levels[classIndex], 1)

	bonus = 0 # ignore high-stat granted bonus spell slots
	for table in MageTable, ClericTable:
		if table == "*":
			continue

		booktype = CommonTables.ClassSkills.GetValue (BaseClassName, "SPLTYPE")
		# set our memorizable counts
		Spellbook.SetupSpellLevels (pc, table, booktype, level)

		if table == MageTable:
			continue

		# learn all our priest spells up to the level we can learn
		for slevel in range (9):
			if GemRB.GetMemorizableSpellsCount (pc, booktype, slevel, bonus) <= 0:
				# actually checks level+1 (runs if level-1 has memorizations)
				Spellbook.LearnPriestSpells (pc, slevel, booktype)
				break

		# FIXME: healing spell hack until we have a separate memo window/phase
		idx = Spellbook.HasSpell (pc, booktype, 0, "sppr103")
		if idx != -1:
			GemRB.MemorizeSpell (pc, booktype, 0, idx, 1)

		# set up shapes for druids
		if not chargen and booktype == IE_IWD2_SPELL_DRUID:
			ShapeTable = GemRB.LoadTable ("mxdrdshp")
			newUses = ShapeTable.GetValue (str(level), "USES_PER_DAY")
			booktype = IE_IWD2_SPELL_SHAPE
			extras = GemRB.HasFeat (pc, FEAT_EXTRA_SHAPESHIFTING)
			GemRB.SetMemorizableSpellsCount (pc, newUses+extras, booktype, 0)

		if booktype != IE_IWD2_SPELL_CLERIC:
			return

		# learn any domain spells too
		booktype = IE_IWD2_SPELL_DOMAIN
		Spellbook.SetupSpellLevels (pc, table, booktype, level)

		for slevel in range (9):
			if GemRB.GetMemorizableSpellsCount (pc, booktype, slevel, bonus) <= 0:
				# actually checks level+1 (runs if level-1 has memorizations)
				Spellbook.LearnPriestSpells (pc, slevel, booktype, BaseClassName)
				break

def SetSpell(pc, SpellName, Feat):
	if GemRB.HasFeat (pc, Feat):
		MakeSpellCount(pc, SpellName, 1)
	else:
		GemRB.RemoveSpell(pc, SpellName)
	return

def MakeSpellCount (pc, spell, count):
	have = GemRB.CountSpells (pc, spell, -1, 1)
	if count <= have:
		return
	# only used for innates, which are all level 1
	Spellbook.LearnSpell (pc, spell, IE_IWD2_SPELL_INNATE, 0, count - have, LS_MEMO)
	return

def SetSpellFocus (pc):
	SPLFocusTable = GemRB.LoadTable ("splfocus")
	for i in range(SPLFocusTable.GetRowCount()):
		Row = SPLFocusTable.GetRowName (i)
		Stat = SPLFocusTable.GetValue (Row, "STAT", GTV_STAT)
		if not Stat:
			continue
		Column = GemRB.GetPlayerStat (pc, Stat)
		if not Column:
			continue
		bonus = SPLFocusTable.GetValue (i, Column)
		if not bonus:
			continue
		# ok, so we have the feat and it does have a bonus assigned in the table
		# make sure we don't duplicate the effect, since it is cummulative
		focused = GemRB.CountEffects (pc, "SpellFocus", bonus, i)
		if focused:
			continue
		# what if we're upgrading to greater spell focus? Remove and readd
		focused = GemRB.CountEffects (pc, "SpellFocus", -1, i)
		if focused:
			GemRB.DispelEffect (pc, "SpellFocus", i)
		GemRB.ApplyEffect (pc, "SpellFocus", bonus, i, "", "", "", "SPLFOCUS", 1)

# make sure this function remains idempotent
def LearnFeatInnates (pc, party, setup):
	global FeatSPLs

	# npcs don't have these feat spells yet, eg. 00solbas is missing power attack
	# party members are handled more sanely during cg/level up
	if setup or not party:
		SetSpell (pc, "SPIN111", FEAT_WILDSHAPE_BOAR)
		SetSpell (pc, "SPIN197", FEAT_MAXIMIZED_ATTACKS)
		SetSpell (pc, "SPIN231", FEAT_ENVENOM_WEAPON)
		SetSpell (pc, "SPIN245", FEAT_WILDSHAPE_PANTHER)
		SetSpell (pc, "SPIN246", FEAT_WILDSHAPE_SHAMBLER)
		SetSpell (pc, "SPIN275", FEAT_POWER_ATTACK)
		SetSpell (pc, "SPIN276", FEAT_EXPERTISE)
		SetSpell (pc, "SPIN277", FEAT_ARTERIAL_STRIKE)
		SetSpell (pc, "SPIN278", FEAT_HAMSTRING)
		SetSpell (pc, "SPIN279", FEAT_RAPID_SHOT)
		LearnExtraFeats (pc)

		# recheck if we need to do anything about new spell focus feats
		SetSpellFocus (pc)

	# apply the rest on area entry
	if setup == True:
		return

	# cache feat spell existence checks, since this gets called on every actor
	if not FeatSPLs:
		for i in range(96): # MAX_FEATS
			featSPL = "FEAT{:02x}".format(i)
			if GemRB.HasResource (featSPL, RES_SPL, 1):
				FeatSPLs.append (featSPL)

	# feats with spell payloads
	# these spells have non-permanent timing, don't survive saves, so
	# we have to reapply them on everyone
	for i in range(96): # MAX_FEATS
		level = GemRB.HasFeat (pc, i)
		if not level:
			continue
		featSPL = "FEAT{:02x}".format(i)
		if featSPL not in FeatSPLs:
			continue
		# make sure it's not applied already if called from level up
		if setup != 2 or GemRB.CountEffects (pc, "", -1, -1, "", featSPL) == 0:
			GemRB.ApplySpell (pc, featSPL, pc)

# make sure this function remains idempotent
def LearnExtraFeats (MyChar):
	# extra rage
	level = GemRB.GetPlayerStat (MyChar, IE_LEVELBARBARIAN)
	if level > 0:
		if level >= 15:
			GemRB.RemoveSpell (MyChar, "SPIN236")
			Spell = "SPIN260"
		else:
			GemRB.RemoveSpell (MyChar, "SPIN260")
			Spell = "SPIN236"
		cnt = GemRB.GetPlayerStat (MyChar, IE_FEAT_EXTRA_RAGE) + (level + 3) // 4
		MakeSpellCount (MyChar, Spell, cnt)
	else:
		GemRB.RemoveSpell(MyChar, "SPIN236")
		GemRB.RemoveSpell(MyChar, "SPIN260")

	# extra smiting
	level = GemRB.GetPlayerStat(MyChar, IE_LEVELPALADIN)
	if level > 1:
		cnt = GemRB.GetPlayerStat (MyChar, IE_FEAT_EXTRA_SMITING) + 1
		MakeSpellCount (MyChar, "SPIN152", cnt)
	else:
		GemRB.RemoveSpell (MyChar, "SPIN152")

	# extra turning
	level = GemRB.GetPlayerStat (MyChar, IE_TURNUNDEADLEVEL)
	if level>0:
		cnt = GUICommon.GetAbilityBonus (MyChar, IE_CHR) + 3
		if cnt<1: cnt = 1
		cnt += GemRB.GetPlayerStat (MyChar, IE_FEAT_EXTRA_TURNING)
		MakeSpellCount (MyChar, "SPIN970", cnt)
	else:
		GemRB.RemoveSpell (MyChar, "SPIN970")

	# stunning fist
	if GemRB.HasFeat (MyChar, FEAT_STUNNING_FIST):
		cnt = GemRB.GetPlayerStat(MyChar, IE_CLASSLEVELSUM) // 4
		MakeSpellCount (MyChar, "SPIN232", cnt)
	else:
		GemRB.RemoveSpell (MyChar, "SPIN232")



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# Start.py - intro and main menu screens

###################################################

import GemRB
from GUIDefines import *

def OnLoad():
	skip_videos = GemRB.GetVar ("SkipIntroVideos")

	if not skip_videos:
		GemRB.PlayMovie ("BISLOGO", 1)
		GemRB.PlayMovie ("TSRLOGO", 1)
		GemRB.PlayMovie ("OPENING", 1)

		GemRB.SetVar ("SkipIntroVideos", 1)

	StartWindow = GemRB.LoadWindow(0, "START")
	NewLifeButton = StartWindow.GetControl(0)
	ResumeLifeButton = StartWindow.GetControl(2)
	ExitButton = StartWindow.GetControl(3)
	NewLifeButton.OnPress (NewLifePress)
	ResumeLifeButton.OnPress (ResumeLifePress)
	ExitButton.OnPress (QuitPress)
	ExitButton.MakeEscape()

	Label = StartWindow.CreateLabel(0x0fff0000, 0,415,640,30, "FONTDLG", "", IE_FONT_SINGLE_LINE | IE_FONT_ALIGN_CENTER)
	Label.SetText(GemRB.Version)
	Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})

	StartWindow.Focus()

	GemRB.LoadMusicPL("Main.mus")
	return
	
def QuitPress():
	QuitWindow = GemRB.LoadWindow(3, "START")
	QuitTextArea = QuitWindow.GetControl(0)
	QuitTextArea.SetText(20582)
	
	ConfirmButton = QuitWindow.GetControl(1)
	ConfirmButton.SetText(23787)
	ConfirmButton.OnPress (lambda: GemRB.Quit())
	ConfirmButton.MakeDefault()

	CancelButton = QuitWindow.GetControl(2)
	CancelButton.SetText(23789)
	CancelButton.OnPress (QuitWindow.Close)
	CancelButton.MakeEscape()
	
	QuitWindow.ShowModal (MODAL_SHADOW_GRAY)
	return
	
def NewLifePress():
	#to make difference between ingame change and new life
	GemRB.SetVar("PlayMode",0)
	GemRB.SetNextScript("NewLife")
	return

def ResumeLifePress():
	#to make difference between ingame load and initial load
	GemRB.SetVar("PlayMode",0)
	GemRB.SetNextScript("GUILOAD")
	return
	



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2017 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import GemRB
import GUICommon
from GUIDefines import *

def CreateClockButton(Button):
	Button.SetAnimation ("WMTIME", 0, A_ANI_GAMEANIM)
	Button.SetState (IE_GUI_BUTTON_LOCKED)
	Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_SET)
	Button.OnPress (lambda: GemRB.GamePause (2, 0))
	Button.OnMouseEnter (UpdateClock)
	SetPSTGamedaysAndHourToken ()
	Button.SetTooltip (GemRB.GetString(65027))
	
	UpdateClock()

def SetPSTGamedaysAndHourToken ():
	currentTime = GemRB.GetGameTime()
	hours = (currentTime % 7200) / 300
	if hours < 12:
		ampm = "AM"
	else:
		ampm = "PM"
		hours -= 12
	minutes = (currentTime % 300) / 60

	GemRB.SetToken ('CLOCK_HOUR', str (hours))
	GemRB.SetToken ('CLOCK_MINUTE', '%02d' %minutes)
	GemRB.SetToken ('CLOCK_AMPM', ampm)

def UpdateClock ():
	ActionsWindow = GemRB.GetView("ACTWIN")

	SetPSTGamedaysAndHourToken ()
	twin = GemRB.GetView("WIN_TOP")
	Button = ActionsWindow.GetControl (2)

	if twin:
		Button.SetState (IE_GUI_BUTTON_ENABLED)
	else:
		Button.SetState (IE_GUI_BUTTON_DISABLED)



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#
import GemRB
from GUIDefines import *
from GameCheck import HasTOTSC

ExitButton = 0
SinglePlayerButton = 0
MultiPlayerButton = 0
MoviesButton = 0

def OnLoad():
	global ExitButton, MultiPlayerButton, MoviesButton, SinglePlayerButton

	skip_videos = GemRB.GetVar ("SkipIntroVideos")
	if not skip_videos:
		GemRB.PlayMovie ('BG4LOGO',1)
		GemRB.PlayMovie ('TSRLOGO',1)
		GemRB.PlayMovie ('BILOGO',1)
		GemRB.PlayMovie ('INFELOGO',1)
		GemRB.PlayMovie ('INTRO',1)
		GemRB.SetVar ("SkipIntroVideos", 1)

	#main window
	StartWindow = GemRB.LoadWindow (0, "START")
	SinglePlayerButton = StartWindow.GetControl(0)
	MultiPlayerButton = StartWindow.GetControl(1)
	MoviesButton = StartWindow.GetControl(2)
	ExitButton = StartWindow.GetControl(3)

	BackToMain()
	
	GemRB.LoadMusicPL("Theme.mus")
	return
	
def SinglePlayerPress():
	
	SinglePlayerButton.SetText(13728)
	MultiPlayerButton.SetStatus(IE_GUI_BUTTON_ENABLED)
	MultiPlayerButton.SetText(13729)
	ExitButton.SetText(15416)
	MultiPlayerButton.OnPress (LoadSingle)
	SinglePlayerButton.OnPress (NewSingle)
	ExitButton.OnPress (BackToMain)
	ExitButton.MakeEscape()
	MoviesButton.SetText("")
	if HasTOTSC():
		MoviesButton.OnPress (MissionPack)
		MoviesButton.SetText(24110)
	else:
		MoviesButton.SetFlags(IE_GUI_BUTTON_NO_IMAGE, OP_SET)
		MoviesButton.SetStatus(IE_GUI_BUTTON_DISABLED)

	return

def LoadSingle():
	GemRB.SetVar("PlayMode",0)
	GemRB.SetToken ("SaveDir", "save")
	GemRB.SetNextScript("GUILOAD")
	return

def MissionPack():
	GemRB.SetVar("PlayMode",1)
	GemRB.SetToken ("SaveDir", "mpsave")
	GemRB.SetNextScript("GUILOAD")
	return

def NewSingle():
	GemRB.SetVar("PlayMode",0)
	GemRB.SetVar("Slot",1)
	GemRB.LoadGame(None)
	GemRB.SetNextScript("CharGen") #temporarily
	return

def ExitPress():
	QuitWindow = GemRB.LoadWindow (3, "START")

	QuitTextArea = QuitWindow.GetControl (0)
	QuitTextArea.SetText (19532)

	CancelButton = QuitWindow.GetControl (2)
	CancelButton.SetText (13727)
	CancelButton.OnPress (QuitWindow.Close)
	CancelButton.MakeEscape ()

	ConfirmButton = QuitWindow.GetControl (1)
	ConfirmButton.SetText (15417)
	ConfirmButton.OnPress (ExitConfirmed)
	ConfirmButton.MakeDefault ()

	QuitWindow.ShowModal (MODAL_SHADOW_GRAY)
	return
	
def ExitConfirmed():
	GemRB.Quit()
	return

def MoviesPress():
	GemRB.SetNextScript("GUIMOVIE")
	return
	
def BackToMain():
	SinglePlayerButton.SetStatus(IE_GUI_BUTTON_ENABLED)
	MoviesButton.SetStatus(IE_GUI_BUTTON_ENABLED)
	ExitButton.SetStatus(IE_GUI_BUTTON_ENABLED)
	SinglePlayerButton.SetText(15413)
	MultiPlayerButton.SetText(15414)
	MoviesButton.SetText(15415)
	ExitButton.SetText(15417)
	SinglePlayerButton.OnPress (SinglePlayerPress)
	MultiPlayerButton.SetState (IE_GUI_BUTTON_DISABLED)
	MoviesButton.OnPress (MoviesPress)
	ExitButton.OnPress (ExitPress)
	MoviesButton.SetFlags(IE_GUI_BUTTON_NORMAL, OP_SET)
	ExitButton.SetFlags(IE_GUI_BUTTON_NORMAL, OP_SET)
	ExitButton.MakeEscape()

	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003-2004 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# character generation, mage spells (GUICG7)

import GemRB
import GUICommon
import CommonTables
import LUSpellSelection
import Spellbook
from GUIDefines import *
from ie_stats import *

def OnLoad():
	Slot = GemRB.GetVar ("Slot")
	Class = GemRB.GetPlayerStat (Slot, IE_CLASS)
	ClassName = GUICommon.GetClassRowName (Class, "class")
	TableName = CommonTables.ClassSkills.GetValue(ClassName, "MAGESPELL")

	# get our kit
	KitValue = GemRB.GetPlayerStat (Slot, IE_KIT)

	# open up the spell selection window
	# remember, it is pc, table, level, diff, kit, chargen
	IsMulti = GUICommon.IsMultiClassed (Slot, 1)
	Level = GemRB.GetPlayerStat (Slot, IE_LEVEL)
	if IsMulti[0]>1:
		for i in range (1, IsMulti[0]):
			ClassName = GUICommon.GetClassRowName (IsMulti[i], "class")
			if CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL", GTV_STR) != "*":
				Level = GemRB.GetPlayerStat (Slot, IE_LEVEL2+i-1)
			break
	Spellbook.SetupSpellLevels(Slot, TableName, IE_SPELL_TYPE_WIZARD, 1)

	LUSpellSelection.OpenSpellsWindow (Slot, TableName, Level, Level, KitValue, 1,False)

	return



# GemRB - Infinity Engine Emulator
# Copyright (C) 2021 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import GemRB
import GameCheck
import GUIClasses
import GUICommon
import InventoryCommon
from CommonWindow import SetGameGUIHidden, IsGameGUIHidden
from ie_stats import *
from GUIDefines import *

HideOnClose = False
ContainerWindow = None
Container = None
if GameCheck.IsIWD2():
	leftdiv = 5
	ground_size = 10
else:
	leftdiv = 3
	ground_size = 6

def UpdateContainerWindow ():
	global Container

	Window = ContainerWindow

	pc = GemRB.GameGetFirstSelectedPC ()
	if GameCheck.IsPST():
		GUICommon.SetEncumbranceLabels (Window, 54, None, pc)
	else:
		GUICommon.SetEncumbranceLabels (Window, 0x10000045, 0x10000046, pc)

	party_gold = GemRB.GameGetPartyGold ()
	Text = Window.GetControl (0x10000036)
	Text.SetText (str (party_gold))

	Container = GemRB.GetContainer (0) #will use first selected pc anyway
	LeftCount = Container['ItemCount']
	ScrollBar = Window.GetControl (52)
	Count = max (0, (LeftCount - ground_size + leftdiv - 1) // leftdiv)
	ScrollBar.SetVarAssoc ("LeftTopIndex", GemRB.GetVar ("LeftTopIndex"), 0, Count)

	inventory_slots = GemRB.GetSlots (pc, 0x8000)
	RightCount = len(inventory_slots)
	ScrollBar = Window.GetControl (53)
	Count = max (0, (RightCount - 4 + 1) // 2)
	ScrollBar.SetVarAssoc ("RightTopIndex", GemRB.GetVar ("RightTopIndex"), 0, Count)

	RedrawContainerWindow ()

def RedrawContainerWindow ():
	Window = ContainerWindow

	# scroll in multiples of the number of columns
	LeftTopIndex = GemRB.GetVar ("LeftTopIndex") * leftdiv
	RightTopIndex = GemRB.GetVar ("RightTopIndex") * 2
	pc = GemRB.GameGetFirstSelectedPC ()
	inventory_slots = GemRB.GetSlots (pc, 0x8000)
	RightCount = len(inventory_slots)

	for i in range (ground_size):
		#this is an autoselected container, but we could use PC too
		Slot = GemRB.GetContainerItem (0, i+LeftTopIndex)
		Button = Window.GetControl (i)
		if Slot:
			Button.SetVarAssoc ("LeftIndex", LeftTopIndex+i)
			callback = TakeItemContainer
		else:
			Button.SetVarAssoc ("LeftIndex", None)
			callback = None
		if GameCheck.IsPST():
			SetItemButton (Window, Button, Slot, callback, None)
		else:
			InventoryCommon.UpdateInventorySlot (pc, Button, Slot, "container")

	for i in range (4):
		if i+RightTopIndex < RightCount:
			Slot = GemRB.GetSlotItem (pc, inventory_slots[i+RightTopIndex])
		else:
			Slot = None
		Button = Window.GetControl (i+10)

		#pst had a redundant call here, reenable if it turns out it isn't redundant:
		#SetItemButton (Window, Button, Slot, None, None)

		if Slot:
			Button.SetVarAssoc ("RightIndex", RightTopIndex+i)
			callback = DropItemContainer
		else:
			Button.SetVarAssoc ("RightIndex", None)
			callback = None
		if GameCheck.IsPST():
			SetItemButton (Window, Button, Slot, callback, None)
		else:
			InventoryCommon.UpdateInventorySlot (pc, Button, Slot, "inventory")

	# shade the inventory icon if it is full
	Button = Window.GetControl (54)
	if Button:
		free_slots = GemRB.GetSlots (pc, 0x8000, -1)
		if free_slots == ():
			Button.SetState (IE_GUI_BUTTON_PRESSED)
		else:
			Button.SetState (IE_GUI_BUTTON_LOCKED)

def OpenContainerWindow ():
	global ContainerWindow, Container

	if ContainerWindow:
		return

	global HideOnClose
	HideOnClose = IsGameGUIHidden()

	if HideOnClose:
		SetGameGUIHidden(False)
		# must use a timed event because SetGameGUIHidden(False) sets a flag to unhide the gui next frame
		# AFIK not needed, commented out in case it turns out there are conditions we will need it
		# GemRB.SetTimedEvent (lambda: GemRB.GetView ("MSGWIN").SetVisible(False), 1)
	else:
		GemRB.GetView ("MSGWIN").SetVisible (False)
		ActWin = GemRB.GetView ("ACTWIN")
		if ActWin:
			ActWin.SetVisible (False)

	ContainerWindow = Window = GemRB.LoadWindow (8, GUICommon.GetWindowPack(), WINDOW_BOTTOM|WINDOW_HCENTER)
	ContainerWindow.SetEventProxy(GUICommon.GameControl)

	# fix wrong height in the guiw10.chu and reposition
	# that chu is also used as a base for some arbitrary resolutions
	if GameCheck.IsBG2 () and GemRB.GetSystemVariable (SV_HEIGHT) >= 768:
		Size = Window.GetSize ()
		Pos = Window.GetPos ()
		Window.SetSize (Size[0], 90)
		Window.SetPos (Pos[0], GemRB.GetSystemVariable (SV_HEIGHT) - 90)

	if not GameCheck.IsIWD2 () and not GameCheck.IsGemRBDemo () and not GameCheck.IsPST ():
		# container window shouldnt be in front
		GemRB.GetView("OPTWIN").Focus()
		GemRB.GetView("PORTWIN").Focus()

	Container = GemRB.GetContainer(0)

	# Gears (time) when options pane is down
	if GameCheck.IsBG2OrEE ():
		import Clock
		Clock.CreateClockButton(Window.GetControl (62))

	# 0-5 - Ground Item
	for i in range (ground_size):
		Button = Window.GetControl (i)
		Button.SetVarAssoc ("LeftIndex", i)
		Button.OnPress (TakeItemContainer)
		if GameCheck.IsPST():
			Button.SetFont ("NUMBER")
			Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT | IE_GUI_BUTTON_ALIGN_BOTTOM, OP_OR)

	# 10-13 - Personal Item
	for i in range (4):
		Button = Window.GetControl (i+10)
		Button.SetVarAssoc ("RightIndex", i)
		Button.OnPress (DropItemContainer)
		if GameCheck.IsPST():
			Button.SetFont ("NUMBER")
			Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT | IE_GUI_BUTTON_ALIGN_BOTTOM, OP_OR)

	# left scrollbar (container)
	ScrollBar = Window.GetControl (52)
	ScrollBar.SetVisible(True) # unhide because in PST it is linked to a TextArea
	ScrollBar.OnChange (RedrawContainerWindow)

	# right scrollbar (inventory)
	ScrollBar = Window.GetControl (53)
	ScrollBar.SetVisible(True) # unhide because in PST it is linked to a TextArea
	ScrollBar.OnChange (RedrawContainerWindow)

	# encumbrance and inventory icon
	# iwd has a handy button
	Button = Window.GetControl (54)
	if Button:
		if GameCheck.IsPST():
			Button.SetFont ("NUMBER")
			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
		Button.CreateLabel (0x10000045, "NUMBER", "", IE_FONT_ALIGN_LEFT|IE_FONT_ALIGN_TOP|IE_FONT_SINGLE_LINE)
		Button.CreateLabel (0x10000046, "NUMBER", "", IE_FONT_ALIGN_RIGHT|IE_FONT_ALIGN_BOTTOM|IE_FONT_SINGLE_LINE)
	else:
		Window.CreateLabel (0x10000045, 323, 14, 60, 15, "NUMBER", "0:", IE_FONT_ALIGN_LEFT|IE_FONT_ALIGN_TOP|IE_FONT_SINGLE_LINE)
		Window.CreateLabel (0x10000046, 323, 20, 80, 15, "NUMBER", "0:", IE_FONT_ALIGN_RIGHT|IE_FONT_ALIGN_TOP|IE_FONT_SINGLE_LINE)

	# container icon
	Button = Window.GetControl (50)
	if GameCheck.IsPST():
		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
	Button.SetState (IE_GUI_BUTTON_LOCKED)

	if not GameCheck.IsPST():
		Table = GemRB.LoadTable ("containr")
		row = Container['Type']
		tmp = Table.GetValue (row, 0)
		if tmp!='*':
			GemRB.PlaySound (tmp)
		tmp = Table.GetValue (row, 1)
		if tmp!='*':
			Button.SetSprites (tmp, 0, 0, 0, 0, 0 )

	# Done
	Button = Window.GetControl (51)
	if GameCheck.IsPST():
		Button.SetText (1403)
	else:
		Button.SetText ("")

	Button.MakeEscape()
	Button.OnPress (LeaveContainer)

	GemRB.SetVar ("LeftTopIndex", 0)
	GemRB.SetVar ("RightTopIndex", 0)
	UpdateContainerWindow ()

def CloseContainerWindow ():
	global ContainerWindow

	if not ContainerWindow:
		return

	ContainerWindow.Close ()
	ContainerWindow = None
	GemRB.GetView ("MSGWIN").SetVisible (True)
	if GemRB.GetView ("ACTWIN"):
		GemRB.GetView ("ACTWIN").SetVisible (True)
	SetGameGUIHidden(HideOnClose)

	Table = GemRB.LoadTable ("containr")
	row = Container['Type']
	tmp = Table.GetValue (row, 2)
	#play closing sound if applicable
	if tmp!='*':
		GemRB.PlaySound (tmp)

#doing this way it will inform the core system too, which in turn will call
#CloseContainerWindow ()
def LeaveContainer ():
	GemRB.LeaveContainer()

def DropItemContainer ():
	RightIndex = GemRB.GetVar ("RightIndex")
	if RightIndex is None:
		return

	#we need to get the right slot number
	pc = GemRB.GameGetFirstSelectedPC ()
	inventory_slots = GemRB.GetSlots (pc, 0x8000)
	if RightIndex >= len(inventory_slots):
		return

	GemRB.ChangeContainerItem (0, inventory_slots[RightIndex], 0)
	UpdateContainerWindow ()

def TakeItemContainer ():
	LeftIndex = GemRB.GetVar ("LeftIndex")
	if LeftIndex is None:
		return

	if LeftIndex >= Container['ItemCount']:
		return

	GemRB.ChangeContainerItem (0, LeftIndex, 1)
	UpdateContainerWindow ()

# pst container override (maybe recheck if UpdateInventorySlot could handle it)
def SetItemButton (Window, Button, Slot, PressHandler, RightPressHandler):
	if Slot != None:
		Item = GemRB.GetItem (Slot['ItemResRef'])
		identified = Slot['Flags'] & IE_INV_ITEM_IDENTIFIED
		Button.SetItemIcon (Slot['ItemResRef'],0)

		if Item['MaxStackAmount'] > 1:
			Button.SetText (str (Slot['Usages0']))
		else:
			Button.SetText ('')

		if not identified or Item['ItemNameIdentified'] == -1:
			Button.SetTooltip (Item['ItemName'])
		else:
			Button.SetTooltip (Item['ItemNameIdentified'])

		Button.OnPress (PressHandler)
		Button.OnRightPress (RightPressHandler)
	else:
		Button.SetItemIcon ('')
		Button.SetTooltip (4273) # Ground Item
		Button.SetText ('')
		Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)

		Button.OnPress (None)
		Button.OnRightPress (None)



# GemRB - Infinity Engine Emulator
# Copyright (C) 2003 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#


# GUIOPT.py - scripts to control options windows mostly from GUIOPT winpack

# GUIOPT:
# 0 - Main options window (peacock tail)
# 1 - Video options window
# 2 - msg win with 1 button
# 3 - msg win with 2 buttons
# 4 - msg win with 3 buttons
# 5 - Audio options window
# 6 - Gameplay options window
# 8 - Feedback options window
# 9 - Autopause options window

###################################################
import Container
import GemRB
import GUICommon
import GUICommonWindows
import GUISAVE
import GUIOPTControls
from ie_sounds import *
from GUIDefines import *

###################################################
def InitOptionsWindow (Window):
	"""Open main options window (peacock tail)"""

	GemRB.GamePause (1, 1)
	TrySavingConfiguration ()

	Container.CloseContainerWindow ()

	def ConfigOptButton(button, strref, action):
		button.SetText (strref)
		button.OnPress (action)

	# Return to Game
	ConfigOptButton(Window.GetControl (0), 28638, Window.Close)

	# Quit Game
	ConfigOptButton(Window.GetControl (1), 2595, OpenQuitMsgWindow)

	# Load Game
	# german pst has two spaces that need to be squished
	LoadButton = Window.GetControl (2)
	LoadGameString = GemRB.GetString (2592)
	NewString = " ".join(LoadGameString.split())
	LoadButton.SetText (NewString)
	LoadButton.OnPress (OpenLoadMsgWindow)

	# Save Game
	ConfigOptButton(Window.GetControl (3), 20639, GUISAVE.OpenSaveWindow)

	# Video Options
	ConfigOptButton(Window.GetControl (4), 28781, OpenVideoOptionsWindow)

	# Audio Options
	ConfigOptButton(Window.GetControl (5), 29720, OpenAudioOptionsWindow)

	# Gameplay Options
	ConfigOptButton(Window.GetControl (6), 29722, OpenGameplayOptionsWindow)

	# Keyboard Mappings
	ConfigOptButton(Window.GetControl (7), 29723, OpenKeyboardMappingsWindow)

	# Movies
	ConfigOptButton(Window.GetControl (9), 38156, OpenMoviesWindow)

	# game version, e.g. v1.1.0000
	Label = Window.GetControl (0x10000007)
	Label.SetText (GemRB.Version)

	return

ToggleOptionsWindow = GUICommonWindows.CreateTopWinLoader(0, "GUIOPT", GUICommonWindows.ToggleWindow, InitOptionsWindow)
OpenOptionsWindow = GUICommonWindows.CreateTopWinLoader(0, "GUIOPT", GUICommonWindows.OpenWindowOnce, InitOptionsWindow)
	
def TrySavingConfiguration():
	if not GemRB.SaveConfig():
		print("ARGH, could not write config to disk!!")

###################################################

def OpenVideoOptionsWindow ():
	"""Open video options window"""

	def OnClose(Window):
		TrySavingConfiguration()

	Window = GemRB.LoadWindow (1, "GUIOPT")
	Window.AddAlias ("SUB_WIN", 0)
	Window.SetAction(OnClose, ACTION_WINDOW_CLOSED)

	GUIOPTControls.OptHelpText (9, 31052)
	GUIOPTControls.OptDone (Window.Close, 7)
	GUIOPTControls.OptCancel (Window.Close, 8)

	GUIOPTControls.OptSlider (31431, 1, 10, 31234, "Brightness Correction", lambda: GammaFeedback(31431))
	GUIOPTControls.OptSlider (31459, 2, 11, 31429, "Gamma Correction", lambda: GammaFeedback(31459))

	GUIOPTControls.OptCheckbox (31221, 6, 15, 30898, "SoftBlt")
	GUIOPTControls.OptCheckbox (31216, 4, 13, 30896, "SoftMirrorBlt")
	GUIOPTControls.OptCheckbox (31220, 5, 14, 30897, "SoftSrcKeyBlt")

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return
	
def GammaFeedback (feedback_ref):
	GemRB.GetView ("OPTHELP").SetText (feedback_ref)
	GemRB.SetGamma (GemRB.GetVar("Brightness Correction")/5,GemRB.GetVar("Gamma Correction")/20)
	return

###################################################

saved_audio_options = {}

def OpenAudioOptionsWindow ():
	"""Open audio options window"""

	Window = GemRB.LoadWindow (5, "GUIOPT")
	Window.AddAlias ("SUB_WIN", 0)

	def OnClose(Window):
		# Restore values in case of cancel
		if GemRB.GetVar ("Cancel") == 1:
			for k, v in list(saved_audio_options.items ()):
				GemRB.SetVar (k, v)
			UpdateVolume (31210)

		TrySavingConfiguration()

	Window.SetAction(OnClose, ACTION_WINDOW_CLOSED)

	# save values, so we can restore them on cancel
	for v in "Volume Ambients", "Volume SFX", "Volume Voices", "Volume Music", "Volume Movie", "Sound Processing", "Music Processing":
		saved_audio_options[v] = GemRB.GetVar (v)

	GUIOPTControls.OptHelpText (9, 31210)
	GUIOPTControls.OptDone (Window.Close, 7)
	GUIOPTControls.OptCancel (Window.Close, 8)

	GUIOPTControls.OptSlider (31227, 1, 10, 31460, "Volume Ambients", lambda: UpdateVolume(31227))
	GUIOPTControls.OptSlider (31228, 2, 11, 31466, "Volume SFX", lambda: UpdateVolume(31228))
	GUIOPTControls.OptSlider (31226, 3, 12, 31467, "Volume Voices", lambda: UpdateVolume(31226))
	GUIOPTControls.OptSlider (31225, 4, 13, 31468, "Volume Music", lambda: UpdateVolume(31225))
	GUIOPTControls.OptSlider (31229, 5, 14, 31469, "Volume Movie", lambda: UpdateVolume(31229))
	
	GUIOPTControls.OptCheckbox (31224, 6, 15, 30900, "Environmental Audio")
	GUIOPTControls.OptCheckbox (63244, 16, 17, 63242, "Sound Processing")
	GUIOPTControls.OptCheckbox (63247, 18, 19, 63243, "Music Processing")

	Window.ShowModal (MODAL_SHADOW_GRAY)
	
def UpdateVolume (volume_ref):
	helpTA = GemRB.GetView ("OPTHELP")
	if helpTA:
		helpTA.SetText (volume_ref)
	GemRB.UpdateVolume (GEM_SND_VOL_MUSIC)
	GemRB.UpdateVolume (GEM_SND_VOL_AMBIENTS)

###################################################

def OpenGameplayOptionsWindow ():
	"""Open gameplay options window"""

	Window = GemRB.LoadWindow (6, "GUIOPT")
	Window.AddAlias ("SUB_WIN", 0)

	def OnClose(Window):
		TrySavingConfiguration()

	Window.SetAction(OnClose, ACTION_WINDOW_CLOSED)

	GUIOPTControls.OptHelpText (12, 31212)

	GUIOPTControls.OptDone (Window.Close, 10)
	GUIOPTControls.OptCancel (Window.Close, 11)

	GUIOPTControls.OptSlider (31232, 1, 13, 31481, "Tooltips", UpdateTooltips, TOOLTIP_DELAY_FACTOR)
	GUIOPTControls.OptSlider (31230, 2, 14, 31482, "Mouse Scroll Speed", UpdateMouseSpeed)
	GUIOPTControls.OptSlider (31231, 3, 15, 31480, "Keyboard Scroll Speed")
	GUIOPTControls.OptSlider (31233, 4, 16, 31479, "Difficulty Level")

	GUIOPTControls.OptCheckbox (31222, 5, 17, 31217, "Always Dither")
	GUIOPTControls.OptCheckbox (31223, 6, 18, 31218, "Gore")
	GUIOPTControls.OptCheckbox (62419, 22, 23, 62418, "Always Run", GUICommonWindows.ToggleAlwaysRun)

	PSTOptButton (31213, 8, 20, 31478, OpenFeedbackOptionsWindow)
	PSTOptButton (31214, 9, 21, 31470, OpenAutopauseOptionsWindow)

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def UpdateTooltips ():
	GemRB.GetView ("OPTHELP").SetText (31232)
	GemRB.SetTooltipDelay (GemRB.GetVar ("Tooltips") )

def UpdateMouseSpeed ():
	GemRB.GetView ("OPTHELP").SetText (31230)
	GemRB.SetMouseScrollSpeed (GemRB.GetVar ("Mouse Scroll Speed") )

###################################################
	
def OpenFeedbackOptionsWindow ():
	"""Open feedback options window"""

	Window = GemRB.LoadWindow (8, "GUIOPT")
	Window.AddAlias ("SUB_WIN", 1)
	GemRB.SetVar ("Circle Feedback", GemRB.GetVar ("GUI Feedback Level") - 1)

	GUIOPTControls.OptHelpText (9, 31213)
	GUIOPTControls.OptDone (Window.Close, 7)
	GUIOPTControls.OptCancel (Window.Close, 8)

	GUIOPTControls.OptSlider (37411, 1, 10, 37463, "Circle Feedback", UpdateMarkerFeedback)
	GUIOPTControls.OptSlider (37447, 2, 11, 37586, "Locator Feedback Level")
	GUIOPTControls.OptSlider (54878, 20, 21, 54879, "Selection Sounds Frequency")
	GUIOPTControls.OptSlider (54880, 22, 23, 55012, "Command Sounds Frequency")

	# others have: to hit rolls, combat info, [actions], state changes, [selection text], misc
	# pst: states, misc, to hit rolls, combat info, [spell casting]; and separate sliders for selection and command
	# we harmonize it across games by likely breaking compatibility with the original
	# there's no need to disable values 4 and 16 (set by our defaults.ini), since their use takes the separate sliders into account
	GUIOPTControls.OptCheckbox (37460, 6, 15, 37594, 'Effect Text Level', None, 8)
	GUIOPTControls.OptCheckbox (37462, 17, 19, 37596, 'Effect Text Level', None, 32)
	GUIOPTControls.OptCheckbox (37453, 3, 12, 37588, 'Effect Text Level', None, 1)
	GUIOPTControls.OptCheckbox (37457, 4, 13, 37590, 'Effect Text Level', None, 2)
	GUIOPTControls.OptCheckbox (37458, 5, 14, 37592, 'Effect Text Level', None, 64)

	Window.ShowModal (MODAL_SHADOW_GRAY)
	
def UpdateMarkerFeedback ():
	GemRB.SetVar ("GUI Feedback Level", GemRB.GetVar ("Circle Feedback") + 1)

###################################################

def OpenAutopauseOptionsWindow ():
	"""Open autopause options window"""
	
	Window = GemRB.LoadWindow (9, "GUIOPT")
	Window.AddAlias ("SUB_WIN", 1)

	GUIOPTControls.OptHelpText (1, 31214)
	GUIOPTControls.OptDone (Window.Close, 16)
	GUIOPTControls.OptCancel (Window.Close, 17)

	# checkboxes OR the values if they associate to the same variable
	GUIOPTControls.OptCheckbox (37688, 2, 9, 37598, "Auto Pause State", None, 4)
	GUIOPTControls.OptCheckbox (37689, 3, 10, 37681, "Auto Pause State", None, 8)
	GUIOPTControls.OptCheckbox (37690, 4, 11, 37682, "Auto Pause State", None, 16)
	GUIOPTControls.OptCheckbox (37691, 5, 12, 37683, "Auto Pause State", None, 2)
	GUIOPTControls.OptCheckbox (37692, 6, 13, 37684, "Auto Pause State", None, 1)
	GUIOPTControls.OptCheckbox (37693, 7, 14, 37685, "Auto Pause State", None, 32)
	GUIOPTControls.OptCheckbox (37694, 8, 15, 37686, "Auto Pause State", None, 64)

	Window.ShowModal (MODAL_SHADOW_GRAY)

###################################################
###################################################

def OpenLoadMsgWindow ():
	Window = GemRB.LoadWindow (3, "GUIOPT")

	# Load
	Button = Window.GetControl (0)
	Button.SetText (28648)
	Button.OnPress (LoadGame)

	# Cancel
	Button = Window.GetControl (1)
	Button.SetText (4196)
	Button.OnPress (Window.Close)

	# Loading a game will destroy ...
	Text = Window.GetControl (3)
	Text.SetText (39432)

	Window.ShowModal (MODAL_SHADOW_GRAY)


def LoadGame ():
	GemRB.QuitGame ()
	GemRB.SetNextScript ('GUILOAD')



###################################################

def OpenQuitMsgWindow ():
	Window = GemRB.LoadWindow (4, "GUIOPT")

	# Save
	Button = Window.GetControl (0)
	Button.SetText (28645)
	Button.OnPress (SaveGame)
	if GemRB.GetView("GC") is not None:
		Button.SetState (IE_GUI_BUTTON_ENABLED)
	else:
		Button.SetState (IE_GUI_BUTTON_DISABLED)

	# Quit Game
	Button = Window.GetControl (1)
	Button.SetText (2595)
	Button.OnPress (QuitGame)

	# Cancel
	Button = Window.GetControl (2)
	Button.SetText (4196)
	Button.OnPress (Window.Close)
	Button.MakeEscape()

	# The game has not been saved ....
	Text = Window.GetControl (3)
	Text.SetText (39430)  # or 39431 - cannot be saved atm

	Window.ShowModal (MODAL_SHADOW_GRAY)
	return

def QuitGame ():
	GemRB.QuitGame ()
	GemRB.SetNextScript ('Start')

def SaveGame ():
	GemRB.SetVar ("QuitAfterSave", 1)
	GUISAVE.OpenSaveWindow ()

###################################################

key_list = [
	('GemRB', None),
	('Grab pointer', '^G'),
	('Toggle fullscreen', '^F'),
	('Enable cheats', '^T'),
	('', None),
	
	('IE', None),
	('Open Inventory', 'I'),
	('Open Priest Spells', 'P'),
	('Open Mage Spells', 'S'),
	('Pause Game', 'SPC'),
	('Select Weapon', ''),
	('', None),
	]


KEYS_PAGE_SIZE = 60
KEYS_PAGE_COUNT = ((len (key_list) - 1) // KEYS_PAGE_SIZE) + 1

def OpenKeyboardMappingsWindow ():
	global last_key_action

	last_key_action = None
		
	Window = GemRB.LoadWindow (0, "GUIKEYS")

	# Default
	Button = Window.GetControl (3)
	Button.SetText (49051)
	#Button.OnPress (None)

	# Done
	Button = Window.GetControl (4)
	Button.SetText (1403)
	Button.OnPress (Window.Close)
	Button.MakeDefault()

	# Cancel
	Button = Window.GetControl (5)
	Button.SetText (4196)
	Button.OnPress (Window.Close)
	Button.MakeEscape()

	keys_setup_page (Window, 0)

def keys_setup_page (Window, pageno):
	# Page n of n
	Label = Window.GetControl (0x10000001)
	GemRB.SetToken ('PAGE', str (pageno + 1))
	GemRB.SetToken ('NUMPAGES', str (KEYS_PAGE_COUNT))
	Label.SetText (49053)

	for i in range (KEYS_PAGE_SIZE):
		try:
			label, key = key_list[pageno * KEYS_PAGE_SIZE + i]
		except:
			label = ''
			key = None

		if key == None:
			# Section header
			Label = Window.GetControl (0x10000005 + i)
			Label.SetText ('')

			Label = Window.GetControl (0x10000041 + i)
			Label.SetText (label)
			Label.SetColor ({'r' : 0, 'g' : 255, 'b' : 255})
		else:
			Label = Window.GetControl (0x10000005 + i)
			Label.SetText (key)
			#Label.OnPress (lambda: OnActionLabelPress(Window))
			Label.SetVarAssoc ("KeyAction", i)	
			
			Label = Window.GetControl (0x10000041 + i)
			Label.SetText (label)
			#Label.OnPress (lambda: OnActionLabelPress(Window))
			Label.SetVarAssoc ("KeyAction", i)	


last_key_action = None
def OnActionLabelPress (Window):
	global last_key_action
	
	i = GemRB.GetVar ("KeyAction")

	if last_key_action != None:
		Label = Window.GetControl (0x10000005 + last_key_action)
		Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
		Label = Window.GetControl (0x10000041 + last_key_action)
		Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
		
	Label = Window.GetControl (0x10000005 + i)
	Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
	Label = Window.GetControl (0x10000041 + i)
	Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})

	last_key_action = i

	# 49155
	
###################################################

def OpenMoviesWindow ():
	Window = GemRB.LoadWindow (0, "GUIMOVIE")

	# Play Movie
	Button = Window.GetControl (2)
	Button.SetText (33034)
	Button.OnPress (OnPlayMoviePress)

	# Credits
	Button = Window.GetControl (3)
	Button.SetText (33078)
	Button.OnPress (OnCreditsPress)

	# Done
	Button = Window.GetControl (4)
	Button.SetText (1403)
	Button.OnPress (Window.Close)

	# movie list
	List = Window.GetControl (0)
	MovieTable = GemRB.LoadTable ("MOVIDESC")
	GemRB.SetVar ('SelectedMovie', 0)
	List.SetOptions([MovieTable.GetValue (i, 0) for i in range (MovieTable.GetRowCount ())], 'SelectedMovie', -1)

	Window.ShowModal (MODAL_SHADOW_BLACK)

###################################################
def OnPlayMoviePress ():
	selected = GemRB.GetVar ('SelectedMovie')
	
	MovieTable = GemRB.LoadTable ("MOVIDESC")
	key = MovieTable.GetRowName (selected)

	GemRB.PlayMovie (key, 1)

###################################################
def OnCreditsPress ():
	GemRB.PlayMovie ("CREDITS")

###################################################

def PSTOptButton (ctlname, button_id, label_id, label_strref, action):
	"""Standard subwindow button for option windows"""

	window = GemRB.GetView ("SUB_WIN")
	button = window.GetControl (button_id)
	button.OnPress (action)
	help_ta = GemRB.GetView ("OPTHELP")
	button.OnMouseEnter (lambda: help_ta.SetText (ctlname))
	button.OnMouseLeave (lambda: help_ta.SetText (help_ta.Value))

	GUIOPTControls.OptBuddyLabel (label_id, label_strref, ctlname, help_ta.Value)

###################################################
# End of file GUIOPT.py



# GemRB - Infinity Engine Emulator
# Copyright (C) 2010 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import os
import GemRB
from GUIDefines import SV_GAMEPATH

fdict = {}

# Create dict of files in GamePath and GamePath/data
GamePath = GemRB.GetSystemVariable (SV_GAMEPATH)
for file in os.listdir (GamePath):
    ufile = file.upper()
    if ufile == 'DATA':
        for file2 in os.listdir (os.path.join(GamePath, file)):
            fdict[file2.upper()] = 1
    else:
        fdict[ufile] = 1


#
# Return True if all given files/resrefs exist
#
# FILES is a list of tuples NAME, EXTENSION, TYPE
#

def CheckFiles(files):
    for name, ext, ftype in files:
        res = (name + '.' + ext).upper() in fdict or GemRB.HasResource (name, ftype, True)
        if not res:
            return False

    return True

